package com.inforbooks.testbasedatos;

import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.util.Log;

public class AdaptadorBD {
	
	 // nombre de la base de datos
	 private static final String NOMBRE_BD = "empresa";	
	 private static final int VERSION_BD = 1;
	 
	// nombre de la tabla
	 private static final String NOMBRE_TABLA = "empleados";
	 
	// nombres de los campos de la tabla records
	 public static final String BD_ID = "_id";
	 public static final String BD_NOMBRE = "nombre";
	 public static final String BD_NROEMP = "nroemp"; 
	 public static final String BD_SALARIO = "salario"; 
	 
	 private static final String ID_LOG = "USO DE BD";

	 // sentencia SQL de creacin de la tabla
	 private static final String CREATE_BD=
			 		"CREATE TABLE " + NOMBRE_TABLA + 
			 		" (_id integer primary key autoincrement, " +
	        		"nombre TEXT not null, " + 	
	        		"nroemp INTEGER not null, " + 	
			 		"salario REAL not null);";
	        
	 private final Context contexto; 
	 
	 // base de datos de la aplicacin
	 private SQLiteDatabase bdatos;
	 
	 // objeto base de datos auxiliar
	 private AuxiliarBD AuxBD;
	 
	 // constructor de la clase
	 // mtodos de gestin:
	 // open()
	 // close()
	 // insertarFila()
	 // eliminarFila()
	 // leerFila()
	 // actualizarFila()
	 // leerTodos()
	 public AdaptadorBD(Context ctx) {
	        this.contexto = ctx;
	        AuxBD = new AuxiliarBD(contexto);
	 }
	 
	

	 // abre la base de datos
	 public AdaptadorBD open() throws SQLException   {
	 		// Crea o abre una base de datos
	 		// que se usar para lectura y escritura
		
	 		bdatos = AuxBD.getWritableDatabase();
	 		return this;
	 }

	 // cierra la base de datos  
	 public void close()  {
	        AuxBD.close();
	 }
	    
	 // inserta una fila
	 public long insertarFila(String nombre, Integer nroemp, Float salario)   {
	 		// insercin de una fila en la tabla
	 		// formato insert
	 		//	insert(String table, 
	 		//	String nullColumnHack, 
	 		//	ContentValues values)
		  
	 		// devuelve el id de la fila o -1 si no inserta
		 
	 		ContentValues valores = new ContentValues();
	 		valores.put(BD_NOMBRE, nombre);
	 		valores.put(BD_NROEMP, nroemp);
	 		valores.put(BD_SALARIO, salario);
	 		return bdatos.insert(NOMBRE_TABLA, 
	        		null, 
	        		valores);
	 		
	 }

	 // elimina una fila
	 public boolean eliminarFila(Integer id)  {
	 		// elimina filas de la tabla
	 		// formato delete
	 		// 	delete(String table, 
	 		//	String whereClause, 
	 		//	String[] whereArgs)	
				
	 		// devuelve cantidad de filas eliminadas
	 		return bdatos.delete(NOMBRE_TABLA, 
	    		BD_ID + "=" + id, 
	    		null) > 0;
	 }
	 
	 // elimina todas las  filas
	 public boolean eliminarTodo()  {
	 		//  elimina filas de la tabla
	 		//  formato delete
	 		// 	delete(String table, 
	 		//	String whereClause, 
	 		//	String[] whereArgs)	
				
	 		// devuelve cantidad de filas eliminadas
	 		return bdatos.delete(NOMBRE_TABLA, 
	    		null, null) > 0;
	 }

	 // obtiene una fila de un nro de empleado
	 // devuelve un objeto Cursor
	 public Cursor leerPorNroEmpleado(Integer nroemp) throws SQLException  {
	 		// formato del query
	 		// query(boolean distinct,
	 		//		String table, 
	 		//		String[] columns, 
	 		//		String selection, 
	 		//		String[] selectionArgs, 
	 		//		String groupBy, 
	 		//		String having, 
	 		//		String orderBy,
	 		//		String limit)
	 		// Query devuelve un cursor para el conjunto de resultados.
	 		Cursor mCursor =
					bdatos.query(true, 
							NOMBRE_TABLA, 
							new String[] {BD_ID, BD_NOMBRE, BD_NROEMP, BD_SALARIO}, 
							BD_NROEMP + "='" + nroemp +"'",
							null, null, null, 
							BD_SALARIO + " DESC", null);		
		
	 		if (mCursor.getCount() > 0) {
	            mCursor.moveToFirst();
	 		}
	 		return mCursor;
	 	}
	 
	 // obtiene una fila de un empleado
	 // accediendo por nombre
	 // devuelve un objeto Cursor
	 public Cursor leerPorNombre(String nombre) throws SQLException  {
	 		// formato del query
	 		// query(boolean distinct,
	 		//		String table, 
	 		//		String[] columns, 
	 		//		String selection, 
	 		//		String[] selectionArgs, 
	 		//		String groupBy, 
	 		//		String having, 
	 		//		String orderBy,
	 		//		String limit)
	 		// Query devuelve un cursor para el conjunto de resultados.
	 		Cursor mCursor =
					bdatos.query(true, 
							NOMBRE_TABLA, 
							new String[] {BD_ID, BD_NOMBRE, BD_NROEMP, BD_SALARIO}, 
							BD_NOMBRE + "='" + nombre +"'",
							null, null, null, 
							BD_SALARIO + " DESC", null);		
		
	 		if (mCursor.getCount() > 0) {
	            mCursor.moveToFirst();
	 		}
	 		return mCursor;
	 	}

	// actualiza una fila
	public int actualizarFila(Integer id, String nombre, Integer nroemp, Float salario) {
	 		// actualiza filas de la tabla
	 		// formato update
	 		// 	update(String table, 
	 		//	ContentValues values, 
	 		//	String whereClause, 
	 		//	String[] whereArgs)	
		
	 		// devuelve cantidad de filas actualizadas
	 		ContentValues valores = new ContentValues();
	 		valores.put(BD_NOMBRE, nombre);
	 		valores.put(BD_NROEMP, nroemp);
	 		valores.put(BD_SALARIO, salario);
	 		return bdatos.update(NOMBRE_TABLA, valores,	BD_ID + "=" + id, null);
	 	}
	
	// actualiza o inserta una fila  
	public long actualizarPorNombre(String nombre, Integer nroemp, Float salario) {
		 		// actualiza filas de la tabla
		 		// formato update
		 		// 	update(String table, 
		 		//	ContentValues values, 
		 		//	String whereClause, 
		 		//	String[] whereArgs)	
		        Cursor micur = leerPorNroEmpleado(nroemp);
		       
		        if (micur.getCount() > 0){
		        	       	
		        		return actualizarFila(Integer.valueOf(micur.getString(0)),
		        				nombre,
		        				nroemp, 
		        				salario);} 
		        
		        else {
		        // primera vez que registra puntos
				// no tiene filas con ese nombre
		        // por lo que la inserta
		 		
		 			return insertarFila(nombre, nroemp, salario);	
		        }    
	}
		
	
	 	// recupera un conjunto de filas
	 	// ordenado por nombre ascendente 
	 	// devuelve un objeto Cursor
	public Cursor leerTodos()  {
	        return bdatos.query(true,
		        		NOMBRE_TABLA, 
		        		new String[] {BD_ID, BD_NOMBRE, BD_NROEMP, BD_SALARIO}, 
		        		null, null, null, null, 
		        		BD_NOMBRE + " ASC",
		        		"100");
	}
	 // clase auxiliar para crear y mantener la
	 // versin de la base de datos
	 private static class AuxiliarBD extends SQLiteOpenHelper  {
		 AuxiliarBD(Context context) 
		 {
	            super(context, NOMBRE_BD, null, VERSION_BD);
		 }

		 @Override
		 public void onCreate(SQLiteDatabase bd)  {
			 // crea la base de datos 
			 try {bd.execSQL(CREATE_BD);} 
			 catch (SQLException e) {e.printStackTrace();}
		 }
	  
		 @Override
		 public void onUpgrade(SQLiteDatabase bd, int oldVersion, int newVersion)  {
			  // actualiza la versin
			  Log.w(ID_LOG, "Actualiza la versin: " + 
					  oldVersion + 
					  " con la nueva versin: " + 
					  newVersion + 
					  " (esto eliminar los datos!!)");
			  // si existe eliminara la base de datos
			  bd.execSQL("DROP TABLE IF EXISTS " + NOMBRE_TABLA);
			  // recrea la base de datos
		      onCreate(bd);
		  	}
	 }    
}
