package com.inforbooks.tests;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

public class ArchivoCacheActivity extends Activity {
    
	// variables/objetos que se usan en toda la clase
	private EditText editText;
	// tamao del bufer de lectura
	private static final int BUFFER_BYTES = 80;
	
    /** OnCreate: se llama al crearse el objeto. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);      
        
        Button btnGuardar = (Button) findViewById(R.id.btnGuardar);
        Button btnLeer = (Button) findViewById(R.id.btnLeer);   
        
        // se obtiene el control EditText
        // (objeto hijo de TextView pero editable)
        editText = (EditText) findViewById(R.id.editText1);
     
        
        // Un clic en el botn Leer
        // provoca que se lea  
        // el archivo
        btnLeer.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                try 
                {   
                	// Obtener acceso al directorio del cache
                	File cacheDir = getCacheDir();
                	File file = new File(cacheDir, "file01.txt");
                	// acceso al archivo
                    FileInputStream objOfi = new FileInputStream(file);
                	
                    InputStreamReader objIsr = new InputStreamReader(objOfi); 
                  
                    // se crea un bufer de entrada
                    char[] inBuf = new char[BUFFER_BYTES];
                    String str = "";
     
                    // lee el primer bloque
                    // en cLeidos queda el valor de caracteres 
                    // ledos
                    int cLeidos = objIsr.read(inBuf);
                    
                    // lectura bloque a bloque
                    // mientra la cantidad de caracteres ledos
                    // sea mayor a cero
                    while (cLeidos > 0)
                    {                    
                        // copyValueOf: conversin del flujo 
                    	// de caracteres a cadena
                                       	
                    	// atencin: la lectura puede ser menor que 
                    	// la longitud del bufer (es lo normal en 
                    	// la ltima lectura), por eso se usa
                    	// cLeidos y no BUFFER_BYTES
                        str += String.copyValueOf(inBuf, 0, cLeidos);                
                        
                        // se crea el nuevo bufer de entrada
                        inBuf = new char[BUFFER_BYTES];
                       
                        // nueva lectura
                        cLeidos = objIsr.read(inBuf);                       
                    } 
                    // asigna toda la cadena leda
                    // al control editable                                
                     editText.setText(str);
                     
                    // mensaje de lectura correcta
                    Toast.makeText(getBaseContext(),"Lectura correcta del archivo", Toast.LENGTH_SHORT).show();
                    objIsr.close();
} 
                catch (IOException e) { 
               	    // generar un mensaje de excepcin
       			    Log.e("test", "excepcin al procesar archivo en lectura", e);                     
                }            	
            }
        });
    
   
        // Un clic en el botn Guardar
        // provoca que el contenido se escribe en 
        // el archivo
        btnGuardar.setOnClickListener(new View.OnClickListener() {
        public void onClick(View v) {         
        	
        	// se obtiene la cadena de texto
        	// que se han introducido en el
        	// control EditText
        	String str = editText.getText().toString();
            try 
            {   
            	// Obtener acceso al directorio del cache
            	File cacheDir = getCacheDir();
            	
            	File file = new File(cacheDir.getAbsolutePath(), "file01.txt");
            	
            	
            	// acceso al archivo
                FileOutputStream objFos = new FileOutputStream(file);
                
            	        
                
                // se crea un objeto OutputStreamWriter
                // que nos permite escribir
                OutputStreamWriter objOsw = new  OutputStreamWriter(objFos);
                
                
                // escribe la cadena en el archivo
                objOsw.write(str);              
                objOsw.flush(); 
                objOsw.close();
                
                // escritura correcta
                Toast.makeText(getBaseContext(), "Escritura correcta", Toast.LENGTH_SHORT).show();
             
                // limpia el contenido del control
               editText.setText("");
             
             } 
             catch (IOException e) 
             { 
            	 // generar un mensaje de excepcin
    			 Log.e("test", "excepcin al procesar archivo en escritura", e);                     
             }  
             //==================================
        }
    });
    
}
}