package com.inforbooks.audiorecordplayer;

import android.app.Activity;
import android.widget.LinearLayout;
import android.os.Bundle;
import android.os.Environment;
import android.view.ViewGroup;
import android.widget.Button;
import android.view.View;
import android.content.Context;
import android.util.Log;
import android.media.MediaRecorder;
import android.media.MediaPlayer;

import java.io.IOException;


public class MainActivity extends Activity
{
    private static final String LOG = "AudioRecordPlayer";
    private static String mArchivo = null;

    // Objetos MediaRecorder y MediaPlayer
    private MediaRecorder mGrabador = null;
    private MediaPlayer   mReproductor = null;

    // Botones de la interfaz
    private BtnReproducir   mBtnReproducir = null;
    private BtnGrabar 		mBtnGrabar = null;
    
    // Se define el nombre del archivo
    // en el que se registrar la
    // grabacin
    public MainActivity() {
    	// Se obtiene la ruta de acceso
    	// del sistema de archivos del dispositivo
        String path = Environment.getExternalStorageDirectory().getAbsolutePath();
        mArchivo = path + "/audiorecordplayer.3gp";
    }
    
    // Creacin de la actividad
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);     
       
        // Creacin dinmica del layout
        LinearLayout layout = new LinearLayout(this);
        
        // Se crea el botn Grabar
        mBtnGrabar = new BtnGrabar(this);
        
        // Se aade al layout
        layout.addView(mBtnGrabar,
            new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT,
                0));
        
        // Se crea el botn Reproducir
        mBtnReproducir = new BtnReproducir(this); 
        
        // Se aade al layout
        layout.addView(mBtnReproducir,
            new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT,
                0));
        
        // Se asigna el layout a la actividad
        setContentView(layout);
    }

    // Cuando la actividad pasa a pausa
    // se liberan los recursos
    @Override
    public void onPause() {
        super.onPause();
        // Si el objeto no es nulo
        // se liberan los recursos
        // y se asigna null
        if (mGrabador != null) {
            detenerGrabacion();
        }

        // Si el objeto no es nulo
        // se liberan los recursos
        // y se asigna null
        if (mReproductor != null) {
           detenerReproduccion();
        }
    }

    // Liberacin de los objetos 
    // de gestin del audio
    private void detenerGrabacion() {
       	// Se liberan los recursos del objeto
        // y se asigna null
        mGrabador.stop();
        mGrabador.release();
        mGrabador = null;
    }
    
    private void detenerReproduccion() {
    	// Se liberan los recursos del objeto
        // y se asigna null
        mReproductor.release();
        mReproductor = null;
    }
    
    //**********************
    // Clases internas de los botones
    // de la interfaz
    // y gestin de los clicks 
    //***********************
    class BtnGrabar extends Button {
        boolean bGrabando;
        
        public BtnGrabar(Context ctx) {
            super(ctx);
            setText(R.string.grabacion);
            bGrabando = true;
            // define el escuchador
            // del click
            setOnClickListener(clickGrabar);
        }

        OnClickListener clickGrabar = new OnClickListener() {
            public void onClick(View v) {
                tratarGrabacion(bGrabando);
                // Un clic en este botn
                // cuando est grabando 
                // significa colocar texto
                // detener
                if (bGrabando) {
                    setText(R.string.detener_grabacion);
                } else {
                    setText(R.string.grabacion);
                }
                // Invierte el indicador
                // para tener en cuenta en
                // el siguiente clioc
                bGrabando = !bGrabando;
            }
        };

    
    }

    class BtnReproducir extends Button {
        boolean bReproduciendo;  
        
        public BtnReproducir(Context ctx) {
            super(ctx);
            setText(R.string.reproduccion);
            bReproduciendo = true;
            // define el escuchador
            // del click
            setOnClickListener(clickReproducir);
        }

        OnClickListener clickReproducir = new OnClickListener() {
            public void onClick(View v) {
            	// Gestiona la reproduccin
            	// o la detencin de la reproduccin
                tratarReproduccion(bReproduciendo);
                
                // Un clic en este botn
                // cuando est reproduciendo 
                // significa colocar texto
                // Detener
                if (bReproduciendo) {
                    setText(R.string.detener_reproduccion);
                } else {
                    setText(R.string.reproduccion);
                }
                bReproduciendo = !bReproduciendo;
            }
        };
     
    }
    
    //************************
    // Gestin de la grabacin 
    // y reproduccin
    //************************
    private void tratarGrabacion(boolean iniciar) {
    	// Si el indicador es true, inicia;
    	// en caso contrario, detiene
        if (iniciar) iniciarGrabacion();
        else detenerGrabacion();       
    }

    private void tratarReproduccion(boolean iniciar) {
    	// Si el indicador es true, inicia;
    	// en caso contrario, detiene
        if (iniciar) iniciarReproduccion();
        else detenerReproduccion();
        
    }

    //************************
    // Inicio de cada accin
    //************************
    private void iniciarReproduccion() {
    	// Detiene grabacin
    	if (mGrabador != null) {
               detenerGrabacion();
               mBtnGrabar.setText(R.string.grabacion);
        }
    	
        mReproductor = new MediaPlayer();
        // Prepara e inicia la reproduccin
        // desde el archivo utilizado
        // en la grabacin
        try {
            mReproductor.setDataSource(mArchivo);
            mReproductor.prepare();
            mReproductor.start();
        } catch (IOException e) {
            Log.e(LOG, "Error al iniciar reproduccin");
        }
    }

 

    private void iniciarGrabacion() {
    	// Detiene reproduccin
        if (mReproductor != null) {
            detenerReproduccion();
            mBtnReproducir.setText(R.string.reproduccion);
        }
    	
        mGrabador = new MediaRecorder();
        // Define el origen de la grabacin
        // el formato del archivo de salida
        // el archivo de salida
        // y la codificacin
        mGrabador.setOutputFile(mArchivo);
        mGrabador.setAudioSource(MediaRecorder.AudioSource.MIC);
        mGrabador.setOutputFormat(MediaRecorder.OutputFormat.THREE_GPP);        
        mGrabador.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB);

        try {
        	// La preparacin puede
        	// fallar
            mGrabador.prepare();
        } catch (IOException e) {
            Log.e(LOG, "Error al iniciar grabacin");
        }
        // Inicia la grabacin
        mGrabador.start();
    }

   
}
