/*
 * Copyright (C) 2013 Scott McCracken
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *    Adaptacin basada en el archivo: DeviceListActivity.java
 *    del proyecto de ejemplo BluetoothChat
 *    The Android Open Source Project
 */
package com.inforbooks.bluetooth;

import java.util.Set;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

public class BluetoothScanner extends Activity {
	// Clase especfica para realizar 
	// el descubrimiento de dispositivos
	// Bluetooth

	// Mtodos:
	// constructor
	// hacerDescubrimiento()
	// onCreate()
	// ondestroy()

	// Variable de ayuda a la depuracin
	private static final String TAG = "BluetoothScanner";

	// Si true, crea log de debug
	private static final boolean D = true;

	// Datos extras para el objeto Intent
	public static String EXTRA_DIRECCION_DISPOSITIVO = "device_address";

	// Miembros privados de la clase
	private BluetoothAdapter mBtAdapter;
	private ArrayAdapter<String> mEmparejadosArrayAdapter;
	private ArrayAdapter<String> mNuevosArrayAdapter;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		if(D) Log.d(TAG, "+in: onCreate()");

		// Configura la ventana
		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
		setContentView(R.layout.lista_scanner);

		// De modo predeterminado asume CANCELED 
		// por si el usuario sale de la actividad
		setResult(Activity.RESULT_CANCELED);

		// Botn para iniciar el proceso de descubrimiento
		Button scanButton = (Button) findViewById(R.id.button_scan);
		// Click del botn
		scanButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Inicia el proceso de descubrimiento
				hacerDescubrimiento();
				// el botn pasa a ser invisible
				// y que no ocupe espacio
				v.setVisibility(View.GONE);
			}
		});

		// Inicializa los dos adaptadores de arrays
		// uno para los ya emparejados y otro para los nuevos

		mEmparejadosArrayAdapter = new ArrayAdapter<String>(this, R.layout.nombre_dispositivo);
		mNuevosArrayAdapter = new ArrayAdapter<String>(this, R.layout.nombre_dispositivo);

		// Configura un ListView para los dispositivos emparejados
		ListView emparejadosLV = (ListView) findViewById(R.id.paired_devices);
		emparejadosLV.setAdapter(mEmparejadosArrayAdapter);

		// Crea un detector de clics en elementos de la lista
		emparejadosLV.setOnItemClickListener(mDeviceClickListener);

		// Configura un ListView para los dispositivos nuevos
		ListView nuevosLV = (ListView) findViewById(R.id.new_devices);
		nuevosLV.setAdapter(mNuevosArrayAdapter);

		// Crea un detector de clics en elementos de la lista
		nuevosLV.setOnItemClickListener(mDeviceClickListener);


		// Se crea un filtro para la deteccin de dispositivos
		IntentFilter filtro = new IntentFilter(BluetoothDevice.ACTION_FOUND);

		// Se registra un BroadcastReceiver
		this.registerReceiver(mReceptor, filtro);

		// Se registra un BroadcastReceiver para la 
		// finalizacin
		filtro = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
		this.registerReceiver(mReceptor, filtro);

		// Obtiene un adaptador
		mBtAdapter = BluetoothAdapter.getDefaultAdapter();

		// Hay algn dispositivo emparejado?
		Set<BluetoothDevice> dispositivosEmparejados = mBtAdapter.getBondedDevices();

		// Si hay dispositivos emparejados, se aaden a la interfaz
		if (dispositivosEmparejados.size() > 0) {
			findViewById(R.id.title_paired_devices).setVisibility(View.VISIBLE);
			for (BluetoothDevice dispositivo : dispositivosEmparejados) {
				mEmparejadosArrayAdapter.add(dispositivo.getName() + "\n" + dispositivo.getAddress());
			}
		} else {
			// No hay dispositivos emparejados dentro del rango
			// de Bluetooth
			String noDevices = getResources().getText(R.string.none_paired).toString();
			mEmparejadosArrayAdapter.add(noDevices);
		}

	}


	@Override
	protected void onDestroy() {
		super.onDestroy();
		if(D) Log.d(TAG, "+in: onDestroy()");

		// Deshabilita el proceso de descubrimiento
		// nos aseguramos que esto se haga
		// ya adems que consume muchos recursos
		if (mBtAdapter != null) {
			mBtAdapter.cancelDiscovery();
		}

		// Anula el registro de BroadcastReceiver
		this.unregisterReceiver(mReceptor);
	}


	// Proceso de descubrimiento de dispositivos Bluetooth
	private void hacerDescubrimiento() {
		if (D) Log.d(TAG, "+in: hacerDescubrimiento()");

		// Coloca en el ttulo que est
		// haciendo la deteccin...
		setProgressBarIndeterminateVisibility(true);
		setTitle(R.string.scanning);

		// Hace visible el subttulo de nuevos dispositivos
		findViewById(R.id.title_new_devices).setVisibility(View.VISIBLE);

		// Se detiene cualquier proceso de
		// descubrimiento activo
		if (mBtAdapter.isDiscovering()) {
			mBtAdapter.cancelDiscovery();
		}

		// Inicia el proceso de deteccin
		if (mBtAdapter.isEnabled())
			mBtAdapter.startDiscovery();
	}


	// Click en algn elemento de las listas
	// de elementos encontrados (emparejado o nuevo)
	private OnItemClickListener mDeviceClickListener = new OnItemClickListener() {
		public void onItemClick(AdapterView<?> av, View v, int arg2, long arg3) {
			if(D) Log.d(TAG, "+in: onItemClick()");

			// Cancela el proceso de descubrimiento
			// porque ya sabemos que vamos a conectar
			mBtAdapter.cancelDiscovery();

			// Se obtiene la direccin MAC del dispositivo
			// seleccionado (los ltimos 17 caracteres 
			// en la vista)
			String info = ((TextView) v).getText().toString();
			String address = info.substring(info.length() - 17);

			// Crea un objeto Intent 
			// para devolver a la actividad
			// invocante (MainActivity) el resultado
			// de esta actividad (la direccin MAC
			// elegida)
			Intent intent = new Intent();
			intent.putExtra(EXTRA_DIRECCION_DISPOSITIVO, address);

			// Resultado de la actividad y fin de la actividad
			setResult(Activity.RESULT_OK, intent);
			finish();
		}
	};

	// BroadcastReceiver est atento a los dispositivos descubiertos
	// y coloca la lista de nombres y direcciones en la lista
	// de la interfaz  
	private final BroadcastReceiver mReceptor = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			if(D) Log.d(TAG, "+in: onReceive()");

			String accion = intent.getAction();

			// El proceso de descubrimiento encuentra un
			// dispositivo (tipo de accin ACTION_FOUND)
			if (BluetoothDevice.ACTION_FOUND.equals(accion)) {
				// Obtiene el objeto BluetoothDevice desde el objeto Intent
				BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
				// Si ya est emparejado, no hace falta hacer nada
				// porque ya est en la lista de la interfaz
				if (device.getBondState() != BluetoothDevice.BOND_BONDED) {
					mNuevosArrayAdapter.add(device.getName() + "\n" + device.getAddress());
				}
				// Termina el proceso de descubrimiento
				// (tipo de accin ACTION_DISCOVERY_FINISHED)
			} else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(accion)) {
				setProgressBarIndeterminateVisibility(false);
				setTitle(R.string.select_device);
				// Si no hay ningn dispositivo descubierto
				// se coloca un texto informativo
				if (mNuevosArrayAdapter.getCount() == 0) {
					String noDevices = getResources().getText(R.string.none_found).toString();
					mNuevosArrayAdapter.add(noDevices);
				}
			}
		}
	};
}
