/*
 * Copyright (C) 2013 Scott McCracken
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *     Adaptacin basada en el archivo: BluetoothChat.java
 *    del proyecto de ejemplo BluetoothChat
 *    The Android Open Source Project
 */
package com.inforbooks.bluetooth;


import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.app.ActionBar;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


public class MainActivity extends Activity {

	// Mtodos:
	// onCreate()
	// onCreateOptionsMenu()
	// onDestroy()
	// onOptionsItemSelected()
	// onPause()
	// onresume()
	// onStart()
	// onStop()
	// conectarDispositivo()
	// configurarChat()
	// enviarMensaje()
	// hacerloDetectable()
	// onActivityResult()

	// adaptador local Bluetooth  
	private BluetoothAdapter mAdaptadorBT = null;

	// Variables para ayuda a la depuracin
	private static final String TAG = "Bluetooth: MainActivity";

	// Variable booleana para activar el log
	// de depuracin a la entrada de cada mtodo
	private static final boolean D = true;

	// Intentos: tipos de acciones  
	private static final int REQUIERE_CONEXION_SEGURA = 1;
	private static final int REQUIERE_CONEXION_INSEGURA = 2;
	private static final int REQUIERE_USAR_BLUETOOTH = 3;

	// Tipos de mensajes enviados desde el handler BluetoothService  
	public static final int MSG_CAMBIO_ESTADO = 1;
	public static final int MSG_LEER = 2;
	public static final int MSG_ESCRIBIR = 3;
	public static final int MSG_NOMBRE_DISPOSITIVO = 4;
	public static final int MSG_TOAST = 5;

	// Nombres claves recibidos desde el handler BluetoothService  
	public static final String NOMBRE_DISPOSITIVO = "device_name";
	public static final String TOAST = "toast";

	// Views del layout
	private ListView mViewConversacion;
	private EditText mOutEditText;
	private Button mBotonEnviar;

	// Nombre del dispositivo conectado
	private String mNombreDispConectado = null;

	// Adaptador Array para el el subproceso de la 
	// conversacin   
	private ArrayAdapter<String> mArrayConversacion;

	// Bfer para los mensajes de salida  
	private StringBuffer mOutStringBuffer;

	// Objeto que representa el servicio
	private BluetoothService mServicio = null;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		if(D) Log.d(TAG, "+in: onCreate()");
		setContentView(R.layout.activity_main);

		// Obtiene el adaptador Bluetooth 
		// por defecto
		mAdaptadorBT = BluetoothAdapter.getDefaultAdapter();

		// Si el adaptador es null es porque Bluetooth no est
		// soportado y no hay nada que hacer
		if (mAdaptadorBT == null) {
			Toast.makeText(this, "Lo siento... Bluetooth no soportado", Toast.LENGTH_LONG).show();
			finish();
			return;
		}
	}

	//**********************
	// Mtodos pblicos (overrides)
	//**********************

	// Men de las opciones
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.opciones, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		Intent serverIntent = null;
		switch (item.getItemId()) {
		case R.id.secure_connect_scan:
			// Inicia la actividad BlueToothScanner
			// para detectar dispositivos seguros  
			serverIntent = new Intent(this, BluetoothScanner.class);
			startActivityForResult(serverIntent, REQUIERE_CONEXION_SEGURA);
			return true;
		case R.id.insecure_connect_scan:
			// Inicia la actividad BlueToothScanner
			// para detectar dispositivos inseguros  
			serverIntent = new Intent(this, BluetoothScanner.class);
			startActivityForResult(serverIntent, REQUIERE_CONEXION_INSEGURA);
			return true;
		case R.id.discoverable:
			// Se hace que el dispositivo sea detectable
			// por otros dispositivos Bluetooth
			hacerloDetectable();
			return true;
		}
		return false;
	}

	@Override
	public void onStart() {
		super.onStart();
		if(D) Log.d(TAG, "+in: onStart()");

		// Si Bluetooth no est habilitado,
		// se solicita que sea habilitado.
		// configurarChat() se invocar en onActivityResult
		if (!mAdaptadorBT.isEnabled()) {
			Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableIntent, REQUIERE_USAR_BLUETOOTH);
			// En caso contrario, configura la sesin de chat
		} else {
			// Configura el chat
			if (mServicio == null) configurarChat();
		}
	}

	@Override
	public synchronized void onResume() {
		super.onResume();
		if(D) Log.d(TAG, "+in: onResume()");

		// Esta verificacin que se realiza en onResume()
		// nos permite detectar los casos en que BT
		// no fue habilitado durante onStart(), por lo que
		// se hace una pausa pra habilitarlo...
		// onResume() se invoca cuando se vuelve de la
		// accin ACTION_REQUEST_ENABLE.
		if (mServicio != null) {
			// Slo en el estado ESTADO_INACTIVO
			// se sabe que ya no se ha comenzado	        	
			if (mServicio.getEstado() == BluetoothService.ESTADO_INACTIVO) {
				// Start the Bluetooth chat services
				mServicio.start();
			}
		}
	}

	@Override
	public synchronized void onPause() {
		super.onPause();
		if(D) Log.d(TAG, "+in: onPause()");
	}

	@Override
	public void onStop() {
		super.onStop();
		if(D) Log.d(TAG, "+in: onStop()");
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		if(D) Log.d(TAG, "+in: onDestroy()");

		// Detiene el servicio
		if (mServicio != null) mServicio.stop();       
	}

	//**********************
	// Mtodos pblicos (no overrides)
	//**********************

	// Se reciben los resultados de los Intent iniciados
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		if(D) Log.d(TAG, "+in: onActivityresult() " + resultCode);

		switch (requestCode) {
		case REQUIERE_CONEXION_SEGURA:
			// Conexin dispositivo seguro
			if (resultCode == Activity.RESULT_OK) {
				conectarDispositivo(data, true);
			}
			break;
		case REQUIERE_CONEXION_INSEGURA:
			// Conexin dispositivo inseguro
			if (resultCode == Activity.RESULT_OK) {
				conectarDispositivo(data, false);
			}
			break;
		case REQUIERE_USAR_BLUETOOTH:
			// Se solicit la habilitacin de 
			// Bluetooth  
			if (resultCode == Activity.RESULT_OK) { 
				// Bluetooth ahora ya est habilitado,
				// por lo que se configura el chat
				configurarChat();
			} else {
				// El usuario no habilit Bluetooth 
				// o hubo un error
				Log.d(TAG, "Bluetooth no est habilitado ");
				Toast.makeText(this, R.string.bt_not_enabled_leaving, Toast.LENGTH_SHORT).show();
				finish();
			}
		}
	}

	//*******************************************
	// Mtodos privados
	//*******************************************

	private void conectarDispositivo(Intent data, boolean secure) {
		if(D) Log.d(TAG, "+in: conectarDispositivo()");

		// Conexin de un dispositivo mediante su direccin
		// MAC
		// Se recupera su direccin desde
		// los datos del objeto Intent
		String address = data.getExtras()
				.getString(BluetoothScanner.EXTRA_DIRECCION_DISPOSITIVO);

		// El adaptador nos devuelve un objeto
		// BluetoothDevice  
		BluetoothDevice device = mAdaptadorBT.getRemoteDevice(address);

		// Intenta conectar mediante el servicio
		// BluetoothService
		mServicio.conectar(device, secure);
	}   

	// Configuracin del Chat
	private void configurarChat() {
		if (D) Log.d(TAG, "+in: setupChat()");

		// Inicializa el adaptador de array para el
		// subproceso de la conversacin 	         
		mArrayConversacion = new ArrayAdapter<String>(this, R.layout.mensaje);
		mViewConversacion = (ListView) findViewById(R.id.in);
		mViewConversacion.setAdapter(mArrayConversacion);

		// Inicializa el campo compuesto con un listener  
		mOutEditText = (EditText) findViewById(R.id.edit_text_out);
		mOutEditText.setOnEditorActionListener(mWriteListener);

		// Inicializa el botn Enviar con un listener 
		// para los clics 
		mBotonEnviar = (Button) findViewById(R.id.button_send);
		mBotonEnviar.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Enva un mensaje usando el contenido
				// del control EditText
				TextView view = (TextView) findViewById(R.id.edit_text_out);
				// Recupera el texto
				String mensaje = view.getText().toString();
				// Enva el mensaje
				enviarMensaje(mensaje);
			}
		});

		// Inicializa el servicio para realizar
		// conexiones Bluetooth
		mServicio = new BluetoothService(this, mHandler);

		// Inicializa el bfer para los mensajes salientes
		mOutStringBuffer = new StringBuffer("");
	}

	// Detecta eventos en el EditText para verificar
	// si se introduce la tecla Intro
	private TextView.OnEditorActionListener mWriteListener =
			new TextView.OnEditorActionListener() {
		public boolean onEditorAction(TextView view, int actionId, KeyEvent event) {
			// Si la accin es un evento key-up 
			// en la tecla Intro, enva el mensaje
			if (actionId == EditorInfo.IME_NULL && event.getAction() == KeyEvent.ACTION_UP) {
				String mensaje = view.getText().toString();
				enviarMensaje(mensaje);
			}
			if(D) Log.i(TAG, "*sale de: onEditorAction");
			return true;
		}
	};


	// Enva un mensaje
	// parmetro: la cadena de texto que se enva
	private void enviarMensaje(String mensaje) {
		if(D) Log.d(TAG, "+in: enviarMensaje()");

		// Verifica el estado antes de hacer nada
		if (mServicio.getEstado() != BluetoothService.ESTADO_CONECTADO) {
			// Avisa que no hay conexin
			Toast.makeText(this, R.string.not_connected, Toast.LENGTH_SHORT).show();
			return;
		}

		// Hay algo para enviar?  
		if (mensaje.length() > 0) {
			// Obtiene los bytes
			byte[] enviar = mensaje.getBytes();

			// Usa el servicio para escribir los datos
			mServicio.write(enviar);

			// Limpia el bfer de salida y
			// limpia el campo EditText
			mOutStringBuffer.setLength(0);
			mOutEditText.setText(mOutStringBuffer);
		}
	}


	// Se hace que el dispositivo sea detectable
	private void hacerloDetectable() {
		if(D) Log.d(TAG, "+in: hacerloDetectable()");

		// Si no es detectable se inicia la 
		// actividad para hacerlo detectable
		if (mAdaptadorBT.getScanMode() !=
				BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
			Intent hacerloDetectableIntento = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
			// Slo es detectable por 60 segundos
			// si se coloca 0, significa que ser detectable 
			// sin lmite de tiempo
			hacerloDetectableIntento.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 60);
			// Inicia la actividad
			startActivity(hacerloDetectableIntento);
		}
	}


	// Este handler gestiona la informacin que 
	// llega desde el servicio BluetoothService
	private final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message mensaje) {
			switch (mensaje.what) {
			case MSG_CAMBIO_ESTADO:
				if(D) Log.i(TAG, "MSG_CAMBIO_ESTADO: " + mensaje.arg1);

				switch (mensaje.arg1) {
				case BluetoothService.ESTADO_CONECTADO:
					// Se incluye el nombre del dispositivo conectado
					// (vase la codificacin especial en strings.xml) 
					setStatus(getString(R.string.title_connected_to, mNombreDispConectado));
					// limpia el array
					mArrayConversacion.clear();
					break;
				case BluetoothService.ESTADO_CONECTANDO:
					setStatus(R.string.title_connecting);
					break;
				case BluetoothService.ESTADO_ATENTO:
				case BluetoothService.ESTADO_INACTIVO:
					setStatus(R.string.title_not_connected);
					break;
				}
				break;
			case MSG_ESCRIBIR:
				byte[] outBufer = (byte[]) mensaje.obj;
				// construye una cadena desde el bfer
				String outMensaje = new String(outBufer);
				mArrayConversacion.add("Yo:  " + outMensaje);
				break;
			case MSG_LEER:
				byte[] inBufer = (byte[]) mensaje.obj;
				// Lectura de datos enviados desde el dispositivo remoto;
				// construye una cadena desde los bytes vlidos en el bfer          
				String inMensaje = new String(inBufer, 0, mensaje.arg1);
				// Coloca el nombre del dispositivo como
				// nombre del otro usuario del chat 
				mArrayConversacion.add(mNombreDispConectado+":  " + inMensaje);
				break;
			case MSG_NOMBRE_DISPOSITIVO:
				// Almacena el nombre del dispositivo conectado
				mNombreDispConectado = mensaje.getData().getString(NOMBRE_DISPOSITIVO);
				Toast.makeText(getApplicationContext(), "Conectado a "
						+ mNombreDispConectado, Toast.LENGTH_SHORT).show();
				break;
			case MSG_TOAST:
				Toast.makeText(getApplicationContext(), mensaje.getData().getString(TOAST),
						Toast.LENGTH_SHORT).show();
				break;
			}
		}
	};


	private final void setStatus(int resId) {
		// Obtiene una referencia a la barra de accin
		// de la actividad
		final ActionBar actionBar = getActionBar();
		// Asigna el estado a la barra de accin
		actionBar.setSubtitle(resId);
	}

	private final void setStatus(CharSequence subTitle) {
		// Obtiene una referencia a la barra de accin
		// de la actividad
		final ActionBar actionBar = getActionBar();
		// Asigna el estado a la barra de accin
		actionBar.setSubtitle(subTitle);
	}

}

