package com.inforbooks.camara;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.Date;
import java.text.SimpleDateFormat;

import android.os.Bundle;
import android.os.Environment;
import android.app.Activity;
import android.hardware.Camera;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.FrameLayout;

public class MainActivity extends Activity  {

    private Camera oCamara;
    private VistaPrevia oVistaPrevia;
    private static final String TAG = "Camara";   
  

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Crea la instancia de la cmara  
        oCamara = getCameraInstance();

        // Crea el objeto VistaPrevia  
        oVistaPrevia = new VistaPrevia(this, oCamara);

		// Aade al layout el objeto VistaPrevia
        FrameLayout vistaPrevia= (FrameLayout) 
			findViewById(R.id.vista_previa);
        vistaPrevia.addView(oVistaPrevia);
        
       
        
		final Camera.PictureCallback jpeg = new Camera.PictureCallback() {

            @Override
            public void onPictureTaken(byte[] data, Camera camera) {

                File archivoImagen = getFile();
                if (archivoImagen == null){
                    Log.d(TAG, "Error al crear el archivo de salida. ");
                    return;
                }

                try {
        		// Graba los datos de la imagen en un archivo 
        		// en formato jpeg
                FileOutputStream oFos = new FileOutputStream(archivoImagen);
        		oFos.write(data);
                    oFos.close();
                } catch (FileNotFoundException e) {
                    Log.d(TAG, "Error archivo no encontrado: " + 
        			e.getMessage());
                } catch (IOException e) {
                    Log.d(TAG, "Error de E/S: " + e.getMessage());
                }
            }
        };
        
        // Se crea el escuchador para el clic del botn de la captura
        Button btn = (Button) findViewById(R.id.btn_capturar);
        btn.setOnClickListener(
            new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    // Obtiene la imagen de la cmara
                    oCamara.takePicture(null, null, jpeg);
                }
            }
        );
        
    }
    
    // Obtiene una instancia de la cmara
    public static Camera getCameraInstance(){
        Camera camara = null;
        try {
            camara = Camera.open();  
        }
        catch (Exception e){
            // Cmara no disponible
        	Log.d(TAG, "Cmara no disponible: " + e.getMessage());
        }
        return camara;  
    }
    
    @Override
    protected void onPause() {
    	super.onPause();
    	oCamara.release();
    }
    
    
    // Crea un archivo para guardar la imagen
    private static File getFile(){
        // Verifica que el almacenamiento externo est montado   
    	if (!checkExternalStorageMounted()){
    		Log.d(TAG, "SD no est montada");
            return null;
    	}
    		
        File imagesDir = new File(Environment.getExternalStoragePublicDirectory(
                  Environment.DIRECTORY_PICTURES), "Camara");
       
        // Si no existe la carpeta, la crea
        if (! imagesDir.exists()){
            if (! imagesDir.mkdirs()){
                Log.d(TAG, "No pudo crear la carpeta");
                return null;
            }
        }

        // Crea el nombre para el archivo
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date(0));
        File file;
        file = new File(imagesDir.getPath() + File.separator +
            "IMG_"+ timeStamp + ".jpg");     

        return file;
    }
    
    private static Boolean checkExternalStorageMounted() {
        String state = Environment.getExternalStorageState();
        if (Environment.MEDIA_MOUNTED.equals(state)) 
            return true;
        return false; 
    }
    
}
