package com.inforbooks.camara;

import java.io.IOException;

import android.content.Context;
import android.hardware.Camera;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;


//Clase auxiliar para previsualizar la imagen que 
//entra de la cmara
public class VistaPrevia extends SurfaceView implements
		 SurfaceHolder.Callback {

	private static final String TAG = "VistaPrevia";

	// Objetos privados para gestionar la superfcie y la cmara
	private SurfaceHolder oHolder;
	private Camera oCamara;

	// Constructir de la clase
	public VistaPrevia(Context context, Camera camara) {
     super(context);
		// objeto Camera
     oCamara = camara;

		// SurfaceHolder.Callback para recibir notificaciones
		// de la creacin y destruccin de la superfcie  
     oHolder = getHolder();
     oHolder.addCallback(this);
		oHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
		oHolder.setFixedSize(400, 300);
	
 }

	public void surfaceCreated(SurfaceHolder holder) {
     // La superfcie se ha creado  
     try {
         			oCamara.setPreviewDisplay(holder);
         			oCamara.startPreview();
			// Si es necesario se puede dibujar sobre la
			// vista previa
			//
     } catch (IOException e) {
         Log.d(TAG, "Excepcin de E/S: " + 
			e.getMessage());
     }
 }

 public void surfaceDestroyed(SurfaceHolder holder) {
		// En la actividad principal se liberar 
		//la vista previa de la cmara  
 }

	// Cambios de formato
 public void surfaceChanged(SurfaceHolder holder, int format,
		 int width, int height) {
     // Aqu se incluye cdigo cuando la vista previa 
		// puede rotar o cambiar de tamao
     // Antes de cambiar nada hay que detener la vista previa.

     if (oHolder.getSurface() == null){
       // return sin hacer nada: no existe superfcie
       return;
     }

     // Antes de los cambios se detiene la vista previa
     try {
         oCamara.stopPreview();
     } catch (Exception e){
       // se intent utilizar una vista previa no existente
			Log.d(TAG, "Excepcin: " + 
				e.getMessage());
     }

     // Aqu se incluye el cdigo para los cambios de tamao,
		// rotacin, etc.
		// Los valores que se utilicen deben estar
		// soportados por el dispositivo
		//...  

     // Reinicia la vista previa con los nuevos valores
		// que se hayan asignado  
     try {
         oCamara.setPreviewDisplay(oHolder);
         oCamara.startPreview();

     } catch (Exception e){
         Log.d(TAG, "Error al iniciar la vista previa: " + 
			e.getMessage());
     }
 }
}