package com.inforbooks.controlpersonalizado;

import android.content.Context;
import android.view.View;
import android.content.res.Resources;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.RectF;

import android.util.AttributeSet;

// Clase que extiende la clase View
public class CircleMeterView extends View {
	
	// Variables privadas para usar
	// en onDraw()
	private Paint paintAguja;
	private Paint paintMarcas;
	private Paint paintTexto;
	private Paint paintMeter;
	private int   medidaTexto;
	
	// Valores privados de propiedades get/set
	private float meter;
	private String unidad;
	
	// Propiedades de Rango
	private float rangoSuperior;
	private float rangoInferior;
	private float rangoNeto;
 
	
	// Valor del medidor
	public void setMeter(float _valor) {
		// Verifica que la medicin 
		// no salga de los lmites
		// que no sea mayor al rango superior
		float nuevo_valor = Math.min(_valor,rangoSuperior);
		// que no sea menor al rango inferior cero
		nuevo_valor = Math.max(nuevo_valor,rangoInferior);
		
		
		if (nuevo_valor != meter){
			// Fuerza que se redibuje el control
			// pero slo si cambia el valor
			meter = nuevo_valor;
			invalidate();
		}
	}

	public float getMeter() {
		return meter;
	}
	
	// Unidad del medidor
	public void setUnidad(String _unidad) {
		// Asigna el tipo de unidad del medidor
		// grados, km/h, etc.
		unidad = _unidad;
		
	}
	
	// Para definir el rango
	// del medidor desde el 
	// valor inferior al superior
	public void setRango(float _rangoInferior, float _rangoSuperior) {
		rangoInferior = Math.max(0,_rangoInferior);
		rangoSuperior = Math.max(rangoInferior+1,_rangoSuperior);		
		rangoNeto = rangoSuperior - rangoInferior; 
	}
	
	
	// Constructores
	public CircleMeterView(Context context) {
	    super(context);
	    inicializarControl();
	}

	public CircleMeterView(Context context, AttributeSet attrs) {
	    super(context, attrs);
	    inicializarControl();
	}

	public CircleMeterView(Context context,
	                     AttributeSet ats,
	                     int defaultStyle) {
	    super(context, ats, defaultStyle);
	    inicializarControl();
	}

	// Inicializacin del control
	protected void inicializarControl() {
		setFocusable(true);
		   
		Resources recurso = this.getResources();
		
		// Rango por defecto 0 a 100
		setRango(0, 100);
		
		// medida por defecto 0
		setMeter(0);

		// Objetos para dibujar
		// Fondo del medidor
		paintMeter = new Paint(Paint.ANTI_ALIAS_FLAG);
		paintMeter.setColor(recurso.getColor(R.color.color_fondo));
		paintMeter.setStyle(Paint.Style.FILL_AND_STROKE);

		// Nmeros del medidor
		paintTexto = new Paint(Paint.ANTI_ALIAS_FLAG);
		paintTexto.setColor(recurso.getColor(R.color.color_texto));

		// Aguja del medidor 
		paintAguja = new Paint(Paint.ANTI_ALIAS_FLAG);
		paintAguja.setStrokeWidth(5);
		paintAguja.setColor(recurso.getColor(R.color.color_aguja));
		
		// Marcas del medidor
		paintMarcas = new Paint(Paint.ANTI_ALIAS_FLAG);
		paintMarcas.setStrokeWidth(2);
		paintMarcas.setColor(recurso.getColor(R.color.color_marcas));		

		// Calculo del tamao de un texto
		medidaTexto = (int)paintTexto.measureText("yY");
	
	 }



	// El control ocupar todo el ancho posible
	// del canvas
	@Override
	protected void onMeasure(int anchoDisponible, int alturaDisponible) {
		 
		// La forma del velocmetro es un arco de crculo
		int anchoMedido = ajustarAncho(anchoDisponible);
		int alturaMedida = ajustarAltura(alturaDisponible);

		// Se calcula el valor mnimo
		int d = Math.min(anchoMedido, alturaMedida);

		// la altura ser la mitad del ancho 
		setMeasuredDimension(d, d/2);
		
	}

	private int ajustarAncho(int valor) {
		int result = 0;
 
		int mode = MeasureSpec.getMode(valor);
		int size = MeasureSpec.getSize(valor);

		// El objeto contenedor no pone restricciones para
		// el tamao del control hijo
		if (mode == MeasureSpec.UNSPECIFIED) {
			// Se define un valor predeterminado
			result = 200;
		} else {
			// para que ocupe toda el rea posible
			result = size;
		}		
	
		return result;
	}
  
	private int ajustarAltura(int valor) {
		int result = 0;
 
		int mode = MeasureSpec.getMode(valor);
		int size = MeasureSpec.getSize(valor);

		// El objeto contenedor no pone restricciones para
		// el tamao del control hijo
		if (mode == MeasureSpec.UNSPECIFIED) {
			// Se define un valor predeterminado
			result = 100;
		} else {
			// para que ocupe toda el rea posible
			result = size;
		}		
	
		return result;
	}
	// Dibujo del control 
	@Override
	protected void onDraw(Canvas canvas) {
		 
		int anchoMedido = getMeasuredWidth();
		int alturaMedida = getMeasuredHeight();

		// Calcula el punto central
		int px = anchoMedido / 2;
		int py = alturaMedida;
  
		// Calcula el radio
		// como el valor menor
		// entre ancho y alto
		int radio = Math.min(px, py);

		// Dibuja un arco  		
		RectF rectF = new RectF(0,0,2*radio,2*radio);
		canvas.drawArc (rectF, 190, 160, true, paintMeter);
 
		// Guarda la matriz actual 
		// que se coloca en la pila privada
		canvas.save();
		
		// Se dibuja el texto de las unidades
		// del medidor
		// Se calcula el ancho del texto
		float anchoTexto = paintTexto.measureText(unidad);
		// posiciona en el centro en la parte inferior
		canvas.drawText(unidad, px-anchoTexto/2, py-10-medidaTexto, paintTexto);
		
		// Gira 80 grados el arco de crculo
		// (es la mitad de los 160 del arco)
		// para poder empezar a dibujar
		canvas.rotate(-80, px, py);

		// Se hace una marca cada 10 grados
		// y se coloca un texto cada 20 grados
		 
		for (int i = 0; i < 17; i++) {
			//Dibuja la marca 
			canvas.drawLine(px, py-radio, px, py-radio+10, paintMarcas);
			
            // Guarda la matriz
			canvas.save();
			canvas.translate(0, medidaTexto);

			// Dibuja el texto de los
			// nmeros del medidor
			// (se hace cada dos marcas)
			if (i % 2 == 0) {
				// Se calcula la medida que corresponde a cada
				// marca
				// Por ejemplo, en el caso de un velocmetro
				// si el rango es de 240 km/h
				// distribuidos en 160 grados
				// cada 10 grados se marcan 15 km/h
				int v =  (int) (i*10*(rangoNeto)/160);
				// medida en nmeros
				String texto = String.valueOf(v);
				
				// Se calcula el ancho del texto
				anchoTexto = paintTexto.measureText(texto);

				// Se calculan las coordenadas
				int posX = (int)(px-anchoTexto/2);
				int posY = py-radio+medidaTexto;
				
				// Se dibuja el texto
				canvas.drawText(texto, posX, posY, paintTexto);
			}
			// Acompaa la accin save previa 
			canvas.restore();

			// Otro giro de 10 grados
			canvas.rotate(10, px, py);
		}
		// Dibuja la aguja
		// Gira los grados necesarios para colocar
		// la aguja de modo vertical
		canvas.restore();	
		canvas.rotate(-80+(meter*160/(rangoNeto)),px,py);
		// Dibuja la aguja 
		canvas.drawLine(px, py-radio+30, px, py-40, paintAguja);
		
		canvas.restore();
		 
	}
  
}

