package com.inforbooks.downloader;

import android.app.Activity;
import android.app.DownloadManager;
import android.app.DownloadManager.Query;
import android.app.DownloadManager.Request;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.Button;

public class MainActivity extends Activity {

	private static final String TAG = "Downloader";
	private long referenciaDescarga;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);		    
	 
		// Botn 1 para iniciar la descarga
	    Button btnDownload = (Button)findViewById(R.id.button1);
	    btnDownload.setOnClickListener(new Button.OnClickListener() {
	      public void onClick(View v) {	    	 
	        controlarDescarga();
	        descargarArchivo(); 
	      }
	    });
	    
	    // Botn 2 para consultar el estado
	    Button btnQuery = (Button)findViewById(R.id.button2);
	    btnQuery.setOnClickListener(new Button.OnClickListener() {
	      public void onClick(View v) {	    	
	        consultarEstado(); 
	      }
	    });

	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_main, menu);
		return true;
	}

	
	private void controlarDescarga() {
		// Servicio de Download Manager  
		String nombreServicio = Context.DOWNLOAD_SERVICE;
		// Se define como final porque se usar
		// en una clase interna
	    final DownloadManager downloadManager = (DownloadManager)getSystemService(nombreServicio);
	
	    // Controla la terminacin de la descarga
	    IntentFilter filtro = new IntentFilter(DownloadManager.ACTION_DOWNLOAD_COMPLETE);
	     
	    // Creamos un receptor del mensaje
	    BroadcastReceiver receptor = new BroadcastReceiver() {
	      @Override
	      public void onReceive(Context contexto, Intent intento) {
	        long referencia = intento.getLongExtra(DownloadManager.EXTRA_DOWNLOAD_ID, -1);
	        // Se filtra que sea de nuestra descarga 
	        if (referenciaDescarga == referencia) {
	          
	          // Creamos el objeto Query 
	          Query myDownloadQuery = new Query();
	          // La consulta es por el identificador de
	          // la descarga
	          myDownloadQuery.setFilterById(referencia);
	          
	          // La consulta devuelve el objeto Cursor
	          // que contiene todos los detalles
	          // sobre la descarga
	          Cursor miCursor = downloadManager.query(myDownloadQuery);
	          if (miCursor.moveToFirst()) {
	        	  // Se obtienen los ndices de los campos
	        	  // que se quieren extraer
	            int indNombreArchivo = 
	              miCursor.getColumnIndex(DownloadManager.COLUMN_LOCAL_FILENAME);
	            int indUri = 
	              miCursor.getColumnIndex(DownloadManager.COLUMN_LOCAL_URI);
	            int indBytes = 
	  	              miCursor.getColumnIndex(DownloadManager.COLUMN_TOTAL_SIZE_BYTES);

	            String nombreArchivo = miCursor.getString(indNombreArchivo);
	            String uriArchivo = miCursor.getString(indUri);
	            String bytesArchivo = miCursor.getString(indBytes);
	        
	            // Se muestra en el log.
	            Log.i(TAG, nombreArchivo + " (fin de descarga): " + uriArchivo + ":" + bytesArchivo);
	          }
	          miCursor.close();

	        }
	      }
	    };
	    // Se registra para la emisin de la
	    // accin ACTION_DOWNLOAD_COMPLETE
	    registerReceiver(receptor, filtro);
	  }
   
	  
	  private void descargarArchivo() {
	    
		// Servicio de Download Manager  
		String nombreServicio = Context.DOWNLOAD_SERVICE;
		DownloadManager downloadManager;
		downloadManager = (DownloadManager)getSystemService(nombreServicio);

	    Uri uri = Uri.parse("http://www.inforbooks.com/llibresonline_desc.php?id=84-96097-34-X&s=b");
	    DownloadManager.Request request = new Request(uri);
	    
	    // Se guardar la descarga con el nombre "php5.zip"
	    String nombreArchivoExterno = "php5.zip";
	    request.setDestinationInExternalPublicDir(Environment.DIRECTORY_DOWNLOADS, nombreArchivoExterno);
	    
	    // esta opcin requiere nivel API 11
	    // permite que la aplicacin Descargas
	    // del sistema sepa que existe esta descarga
	    request.allowScanningByMediaScanner();
	    
	    request.setTitle(nombreArchivoExterno);
	    // La notificacin ser visible mientras 
	    // se produce la descarga
	    request.setNotificationVisibility(Request.VISIBILITY_VISIBLE);
	    	    
	    // Guardamos el identificador de la descarga
	    referenciaDescarga = downloadManager.enqueue(request);
	    
	    // Se informa en el log
	    Log.i(TAG, "Referencia de la nueva descarga: " + referenciaDescarga);
	  }
	  	  
	  private void consultarEstado() {

	    // Servicio de Download Manager  
		String nombreServicio = Context.DOWNLOAD_SERVICE;
	    DownloadManager downloadManager;
	    downloadManager = (DownloadManager)getSystemService(nombreServicio);

	    // Se consultan varios estados  
	    Query consultaEstado = new Query();
	    int Flags = DownloadManager.STATUS_PENDING + 
	    			DownloadManager.STATUS_FAILED + 
	    			DownloadManager.STATUS_PAUSED + 
	    			DownloadManager.STATUS_SUCCESSFUL;
	     
	    consultaEstado.setFilterByStatus(Flags);

	    // Consulta de las descargas en estado 
	    Cursor cursorConsulta = downloadManager.query(consultaEstado);

	    // Obtiene los ndices del objeto Cursor
	    // de las columnas que queremos mostrar
	    // en el informe del estado
	    int indStatus = cursorConsulta.getColumnIndex(DownloadManager.COLUMN_STATUS);
	    int indRazon = cursorConsulta.getColumnIndex(DownloadManager.COLUMN_REASON);
	    int indTitulo = cursorConsulta.getColumnIndex(DownloadManager.COLUMN_TITLE);
	    int indBytes = 
	      cursorConsulta.getColumnIndex(DownloadManager.COLUMN_TOTAL_SIZE_BYTES);    
	    int indBytesDescargados = 
	      cursorConsulta.getColumnIndex(DownloadManager.COLUMN_BYTES_DOWNLOADED_SO_FAR);

	    // El objeto Cursor tiene todas las descargas
	    // filtradas (0, 1 o ms) 
	    while (cursorConsulta.moveToNext()) {
	      // Se extraen los datos del objeto
	      String titulo = cursorConsulta.getString(indTitulo);
	      int bytes = cursorConsulta.getInt(indBytes);
	      int bytesDescargados = cursorConsulta.getInt(indBytesDescargados);
	      String infoAdicional = "";
	      // Si el estado no es completado
	      if (DownloadManager.STATUS_SUCCESSFUL != indStatus){
		      int razon = cursorConsulta.getInt(indRazon);		      
		      switch (razon) {
		        case DownloadManager.PAUSED_QUEUED_FOR_WIFI : 
		        	infoAdicional = "En cola de espera de red WiFi"; break;
		        case DownloadManager.PAUSED_WAITING_FOR_NETWORK : 
		        	infoAdicional = "En espera de la conexin"; break;
		        case DownloadManager.PAUSED_WAITING_TO_RETRY :
		        	infoAdicional = "En espera para reintentarlo"; break;
		        default : break;
		      }
	     }
	      // Resumen del estado
	      StringBuilder objSB = new StringBuilder();
	      objSB.append(titulo).append("\n");
	      objSB.append(infoAdicional).append("\n");
	      objSB.append("Bytes ").append(bytesDescargados).append(" de un total de " ).append(bytes);

	      // Muestra el estado en el log
	      Log.i(TAG, objSB.toString());	     
	    }

	    Log.i(TAG, "Fin consulta");
	    // Cierre del cursor
	    cursorConsulta.close();
	  }
	  

}
