package com.inforbooks.geociencia.p01;


import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.util.Log;

public class AdaptadorBD {
	
	 // nombre de la base de datos
	 private static final String NOMBRE_BD = "geociencia";	
	 private static final int VERSION_BD = 2;
	 
	// nombre de la tabla
	 private static final String NOMBRE_TABLA = "records";
	 
	// nombres de los campos de la tabla records
	 public static final String BD_ID = "_id";
	 public static final String BD_NOMBRE = "nombre";
	 public static final String BD_PUNTOS = "puntos"; 
	 
	 private static final String ID_LOG = "Gestor de base de datos";

	 // sentencia SQL de creacin de la tabla
	 private static final String CREATE_BD=
			 		"create table " + NOMBRE_TABLA + " (_id integer primary key autoincrement, " +
	        		"nombre text not null, " + 	"puntos numeric not null);";
	        
	 private final Context contexto; 
	 
	 // base de datos de la aplicacin
	 private SQLiteDatabase bdatos;
	 
	 // objeto base de datos auxiliar
	 private AuxiliarBD AuxBD;
	 
	 // constructor de la clase
	 // mtodos de gestin:
	 // open()
	 // close()
	 // insertarFila()
	 // eliminarFila()
	 // leerFila()
	 // actualizarFila()
	 // leerTodos()
	 public AdaptadorBD(Context ctx) {
	        this.contexto = ctx;
	        AuxBD = new AuxiliarBD(contexto);
	 }
	 
	 // clase auxiliar para crear y mantener la
	 // versin de la base de datos
	 private static class AuxiliarBD extends SQLiteOpenHelper  {
		 AuxiliarBD(Context context) 
		 {
	            super(context, NOMBRE_BD, null, VERSION_BD);
		 }

		 @Override
		 public void onCreate(SQLiteDatabase bd)  {
			 // crea la base de datos 
			 try {bd.execSQL(CREATE_BD);} 
			 catch (SQLException e) {e.printStackTrace();}
		 }
	  
		 @Override
		 public void onUpgrade(SQLiteDatabase bd, int oldVersion, int newVersion)  {
			  // actualiza la versin
			  Log.w(ID_LOG, "Actualiza la versin: " + 
					  oldVersion + 
					  " con la nueva versin: " + 
					  newVersion + 
					  " (esto eliminar los datos!!)");
			  // si existe elimina la base de datos
			  bd.execSQL("DROP TABLE IF EXISTS " + NOMBRE_TABLA);
			  // recrea la base de datos
		      onCreate(bd);
		  	}
	 	}    

	 // abre la base de datos
	 public AdaptadorBD open() throws SQLException   {
	 		// Crea o abre una base de datos
	 		// que se usar para lectura y escritura
		
	 		bdatos = AuxBD.getWritableDatabase();
	 		return this;
	 }

	 // cierra la base de datos  
	 public void close()  {
	        AuxBD.close();
	 }
	    
	 // inserta una fila
	 public long insertarFila(String nombre, Integer puntos)   {
	 		// insercin de una fila en la tabla
	 		// formato insert
	 		//	insert(String table, 
	 		//	String nullColumnHack, 
	 		//	ContentValues values)
		
	 		// devuelve el id de la fila o -1 si no inserta
		 
	 		ContentValues valores = new ContentValues();
	 		valores.put(BD_NOMBRE, nombre);
	 		valores.put(BD_PUNTOS, puntos);
	 		return bdatos.insert(NOMBRE_TABLA, 
	        		null, 
	        		valores);
	 }

	 // elimina una fila
	 public boolean eliminarFila(Integer id)  {
	 		// elimina filas de la tabla
	 		// formato delete
	 		// 	delete(String table, 
	 		//	String whereClause, 
	 		//	String[] whereArgs)	
				
	 		// devuelve cantidad de filas eliminadas
	 		return bdatos.delete(NOMBRE_TABLA, 
	    		BD_ID + "=" + id, 
	    		null) > 0;
	 }
	 
	 // elimina todas las  filas
	 public boolean eliminarTodo()  {
	 		// elimina filas de la tabla
	 		// formato delete
	 		// 	delete(String table, 
	 		//	String whereClause, 
	 		//	String[] whereArgs)	
				
	 		// devuelve cantidad de filas eliminadas
	 		return bdatos.delete(NOMBRE_TABLA, 
	    		null, null) > 0;
	 }

	 // obtiene una fila de un nombre determinado
	 // devuelve un objeto Cursor
	 public Cursor leerPorNombre(String nombre) throws SQLException  {
	 		// formato del query
	 		// query(boolean distinct,
	 		//		String table, 
	 		//		String[] columns, 
	 		//		String selection, 
	 		//		String[] selectionArgs, 
	 		//		String groupBy, 
	 		//		String having, 
	 		//		String orderBy,
	 		//		String limit)
	 		// Query devuelve un cursor para el conjunto de resultados.
	 		Cursor mCursor =
					bdatos.query(true, 
							NOMBRE_TABLA, 
							new String[] {BD_ID, BD_NOMBRE, BD_PUNTOS}, 
							BD_NOMBRE + "='" + nombre +"'",
							null, null, null, 
							BD_PUNTOS + " DESC", null);		
		
	 		if (mCursor.getCount() > 0) {
	            mCursor.moveToFirst();
	 		}
	 		return mCursor;
	 	}

	// actualiza una fila
	public int actualizarFila(Integer id, String nombre, Integer puntos) {
	 		// actualiza filas de la tabla
	 		// formato update
	 		// 	update(String table, 
	 		//	ContentValues values, 
	 		//	String whereClause, 
	 		//	String[] whereArgs)	
		
	 		// devuelve cantidad de filas actualizadas
	 		ContentValues campos = new ContentValues();
	 		campos.put(BD_NOMBRE, nombre);
	 		campos.put(BD_PUNTOS, puntos);
	 		return bdatos.update(NOMBRE_TABLA, campos,	BD_ID + "=" + id, null);
	 	}
	
	// actualiza una fila por nombre de jugador
	public long actualizarPorNombre(String nombre, Integer puntos) {
		 		// actualiza filas de la tabla
		 		// formato update
		 		// 	update(String table, 
		 		//	ContentValues values, 
		 		//	String whereClause, 
		 		//	String[] whereArgs)	
		        Cursor micur = leerPorNombre(nombre);
		       
		        if (micur.getCount() > 0){
		        	// compara si hay mejora del record
		        	if (puntos > Integer.valueOf(micur.getString(2)) ){
		        		// reemplaza el valor de los puntos
		        		// devuelve cantidad de filas actualizadas
		        		return actualizarFila(Integer.valueOf(micur.getString(0)), 
		        				micur.getString(1), 
		        				puntos);} 
		        	else {
		        		 // no supera su record anterior
		        		return 0;} } 
		        else {
		        // primera vez que registra puntos
				// no tiene filas con ese nombre
		        // por lo que la inserta
		 		
		 			return insertarFila(nombre, puntos);	
		        }    
	}
		
	
	 	// recupera un conjunto de filas
	 	// ordenado de modo descendente
	 	// devuelve un objeto Cursor
	public Cursor leerTodos()  {
	        return bdatos.query(true,
		        		NOMBRE_TABLA, 
		        		new String[] {BD_ID, BD_NOMBRE, BD_PUNTOS}, 
		        		null, null, null, null, 
		        		BD_PUNTOS + " DESC",
		        		"100");
	}
}
