package com.inforbooks.geociencia.p01;

import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;


import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import org.xmlpull.v1.XmlPullParserException;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.res.AssetManager;
import android.content.res.XmlResourceParser;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageSwitcher;
import android.widget.ImageView;
import android.widget.TextSwitcher;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ViewSwitcher;



public class JugarAct extends GeoCienciaAct {
	
	// objeto de preferencias compartidas
	SharedPreferences oDatosUsuario;

	// objeto Hashtable para almacenar
	// y gestionar la lista de preguntas
	// que se obtienen del archivo XML
	Hashtable<Integer, Pregunta> oListaPreguntas;
	
	// estas variables privadas contienen
	// los datos de cada objeto Pregunta
	private TextSwitcher 	oTextoPregunta;
	private ImageSwitcher 	oImagenPregunta;
	
	private String 			oRespuestaPregunta;
	private Integer 		oValorPregunta;
	private Integer         intPreguntasCargadas = 0;	
		
	@Override
	public void onCreate(Bundle savedInstanceState) {
			super.onCreate(savedInstanceState);
			setContentView(R.layout.jugar);
			
			// Se recuperan los datos de usuario desde 
			// SharedPreferences
	        oDatosUsuario = getSharedPreferences(PREFERENCIAS, Context.MODE_PRIVATE);
	        
	        // se accede al nombre del jugador para colocarlo en la interfaz
	        // (si no hay nada almacenado se coloca el texto "jugador"
	        TextView oTVJugador = (TextView) findViewById(R.id.Jugador);
	        oTVJugador.setText(oDatosUsuario.getString(PREFERENCIAS_NOMBRE, "jugador"));	 
	        
	        //////////////////////////////////////////////
	        // desde qu punto se siguen las preguntas?
	        // Carga las preguntas desde el punto en 
	        // donde haba quedado el jugador  
	        /////////////////////////////////////////////
	        int intPreguntaInicial = oDatosUsuario.getInt(PREFERENCIAS_SECUENCIA, 0);

	        // La primera vez se inicializa el valor del
	        // control de la secuencia de las preguntas	       
	        if (intPreguntaInicial == 0 || intPreguntaInicial > CANTIDAD_PREGUNTAS) {
	        	intPreguntaInicial = 1;
	        	Editor editor = oDatosUsuario.edit();
	        	editor.putInt(PREFERENCIAS_SECUENCIA, 1);
	        	editor.commit();   
	        }
			
			
            //////////////////////////////////////////////
            // Se inicializa el objeto hashtable
            ////////////////////////////////////////////// 
            oListaPreguntas = new Hashtable<Integer, Pregunta>(CANTIDAD_PREGUNTAS);
            
            //////////////////////////////////////////////////
	        // se definen las funciones fbrica de los objetos 
	        // TextSwitcher e ImageSwitcher
	        ////////////////////////////////////////////////// 
	        // se obtiene el objeto TextSwitcher
	        // usado en la interfaz 
	        oTextoPregunta = (TextSwitcher) findViewById(R.id.TextSwitcher_TextoPregunta);

	        // se establece la fbrica de objetos de transicin
	        // para el cambio de vistas del objeto TextView (TextSwitcher)
	        oTextoPregunta.setFactory(new CrearTextViewSiguiente());

	        // se obtiene el objetoImageSwitcher
	        oImagenPregunta = (ImageSwitcher) findViewById(R.id.ImageSwitcher_ImagenPregunta);

	        // se establece la fbrica de objetos de transicin
	        // para el cambio de vistas del objeto ImageView (ImageSwitcher)
	        oImagenPregunta.setFactory(new CrearImageViewSiguiente());	

	        ////////////////////////////////////////////////
            // carga de las preguntas desde el archivo XML
	        ////////////////////////////////////////////////
	        cargarPreguntas(intPreguntaInicial);
	        
	        verificarBD(); 	        
  
		}
	
	///////////////////////////////////
	// gestionar el clic en el botn S
	///////////////////////////////////
	public void onClickSi(View v) {
		tratarRespuestaJugador("s");
	}
	
	///////////////////////////////////
	// gestionar el clic en el botn No
	///////////////////////////////////
	public void onClickNo(View v) {
		tratarRespuestaJugador("n");
	}
	
	///////////////////////////////////
	// gestionar el clic en el botn de reset
	///////////////////////////////////
	public void onClickOtroJuego(View v) {
		// el jugador quiere reiniciar todo desde 0
		Editor editor = oDatosUsuario.edit();
		editor.putInt(PREFERENCIAS_SECUENCIA, 1);
		editor.putInt(PREFERENCIAS_PUNTOS, 0);
		editor.putInt(PREFERENCIAS_FALLOS, 0);
		editor.commit();
        
        // carga las preguntas desde la secuencia 1
        cargarPreguntas(1);
        
        // se activan los dos botones de respuesta
        // para que  se puedan hacer ms clics
        accionBotones(true);   
	};
	
		
	////////////////////////////////////////////////////
	// carga las preguntas desde el archivo XML
	///////////////////////////////////////////////////
	public void cargarPreguntas(Integer intPreguntaInicial) {
		// se lee  el archivo de preguntas
        // y se carga el objeto hashtable
        // desde la secuencia en donde haba
        // quedado el jugador
        try {
            leerArchivoPreguntas(intPreguntaInicial);
        } catch (Exception e) {
            Log.e(TAG, "Error en el acceso al archivo XML", e);
        }
		
        // verifica que la carga del archivo 
        // haya sido correcta  
        if (oListaPreguntas.containsKey(intPreguntaInicial) == true) {
        	
        	// se asignan los datos de la pregunta a los
        	// objetos de la interfaz
        	
            // obtiene el texto para el control TextSwitcher
        	oTextoPregunta.setCurrentText(getTextoPregunta(intPreguntaInicial));
        	
        	// almacena la respuesta correcta 
        	// (claro, no se muestra en pantalla!!)
            oRespuestaPregunta = getRespuestaPregunta(intPreguntaInicial);
            
            // almacena el valor en puntos de la respuesta 
            // si la respuesta es acertada, se suma este valor al
            // record del jugador
            oValorPregunta = getValorPregunta(intPreguntaInicial);
            
            // se obtiene la imagen para el control ImageSwitcher
            Drawable oImagen = getImagenPregunta(intPreguntaInicial);
            oImagenPregunta.setImageDrawable(oImagen);
            
            // se actualiza el contador de puntos en pantalla
            TextView oTV = (TextView) findViewById(R.id.TextView_Resultado); 
            
            // se informan los puntos y los fallos acumulados
            oTV.setText(Integer.toString(oDatosUsuario.getInt(PREFERENCIAS_PUNTOS, 0))+
            		" - " + Integer.toString(oDatosUsuario.getInt(PREFERENCIAS_FALLOS, 0)) );  
            
        } else {
            // finaliza el archivo de preguntas 
            finPreguntas();
        }
		
	}
   
	 ///////////////////////////////////////////////////
     // tratamiento de la respuesta del jugador
     // se obtienen los puntos acumulados
     // se verifica si la respuesta es correcta
     // en caso positivo, se acumulan los puntos
     ///////////////////////////////////////////////////
    private void tratarRespuestaJugador(String strRespuesta) {
    	
    	// se recuperan los puntos y los fallos acumulados
        int contadorPuntos = oDatosUsuario.getInt(
        		PREFERENCIAS_PUNTOS, 0);
        int contadorFallos = oDatosUsuario.getInt(PREFERENCIAS_FALLOS, 0);
        
        // se recupera la secuencia de la ltima pregunta 
        int secuenciaPreguntaSiguiente = oDatosUsuario.getInt(PREFERENCIAS_SECUENCIA, 1) + 1;
        
        // se actualiza la secuencia en las preferencias 
        Editor editor = oDatosUsuario.edit();
        editor.putInt(PREFERENCIAS_SECUENCIA, secuenciaPreguntaSiguiente);

        // si la respuesta es acertada
        // se suman los puntos que vale esa 
        // respuesta en el contador
        // y si es un fallo se suma al 
        // contador de fallos
        contadorFallos = contadorFallos + 1;
        if (strRespuesta.equals(oRespuestaPregunta)) {
            editor.putInt(PREFERENCIAS_PUNTOS, contadorPuntos + oValorPregunta);           
        } else {
            editor.putInt(PREFERENCIAS_FALLOS, contadorFallos);
        }
        // se confirman los cambios en SharedPreferences
        editor.commit();
        
        // se actualizan los contadores en pantalla
        TextView oTV = (TextView) findViewById(R.id.TextView_Resultado); 
        
        // se informan los puntos y los fallos acumulados
        oTV.setText(Integer.toString(oDatosUsuario.getInt(PREFERENCIAS_PUNTOS, 0))+
        		" - " + Integer.toString(oDatosUsuario.getInt(PREFERENCIAS_FALLOS, 0)) );   
        
        // se sigue jugando siempre que no haya habido 3 fallos
        if (contadorFallos < 3 ){
        
        	// se comienza con la siguiente pregunta
        	if (oListaPreguntas.containsKey(secuenciaPreguntaSiguiente) == true) {
        		// se actualizan los controles en la interfaz
        		// texto de la pregunta
        		TextSwitcher oTSwitcher = (TextSwitcher) findViewById(R.id.TextSwitcher_TextoPregunta);
        		oTSwitcher.setText(getTextoPregunta(secuenciaPreguntaSiguiente));

        		// imagen de la pregunta
        		ImageSwitcher oISwitcher = (ImageSwitcher) findViewById(R.id.ImageSwitcher_ImagenPregunta);
        		Drawable oImagen = getImagenPregunta(secuenciaPreguntaSiguiente);        		
        		oISwitcher.setImageDrawable(oImagen);
            
        		// almacena la respuesta correcta 
        		oRespuestaPregunta = getRespuestaPregunta(secuenciaPreguntaSiguiente);
            
        		// almacena el valor en puntos de la respuesta 
        		// si la respuesta es acertada, se suma este valor al
        		// record del jugador
        		oValorPregunta = getValorPregunta(secuenciaPreguntaSiguiente);
            
        	} else {
            // final de la lista de preguntas
        		finPreguntas();
        	}
        } else {
        	finJuego();
        }
    }

     ///////////////////////////////////////////////////
     // se configura la pantalla para indicar se perdi
     ///////////////////////////////////////////////////
	private void finJuego() {    	
    	// texto para cuando se ha perdido
        TextSwitcher oTSwitcher = (TextSwitcher) findViewById(R.id.TextSwitcher_TextoPregunta);
        oTSwitcher.setText(getResources().getText(R.string.jugar_fin_juego));
        
        // imagen para cuando se ha perdido
        ImageSwitcher oISwitcher = (ImageSwitcher) findViewById(R.id.ImageSwitcher_ImagenPregunta);
        oISwitcher.setImageResource(R.drawable.finjuego);
        
        // deshabilita los botones
        accionBotones(false);         

    }
    ////////////////////////////////////////////////////////
    // se configura la pantalla para indicar que no quedan 
	// ms preguntas
    ////////////////////////////////////////////////////////
    private void finPreguntas() {
    	String msg;
    	
    	// texto de no hay ms preguntas
        TextSwitcher oTSwitcher = (TextSwitcher) findViewById(R.id.TextSwitcher_TextoPregunta);
        oTSwitcher.setText(getResources().getText(R.string.jugar_fin_preguntas));
        
        ImageSwitcher oISwitcher = (ImageSwitcher) findViewById(R.id.ImageSwitcher_ImagenPregunta);
        oISwitcher.setImageResource(R.drawable.finjuego);

        // aadimos la fila con el record	
        miBD.open(); 
        long q = miBD.actualizarPorNombre(oDatosUsuario.getString(PREFERENCIAS_NOMBRE, "jugador"), 
        		oDatosUsuario.getInt(PREFERENCIAS_PUNTOS, 0));           
           
        if (q > 0)  	msg = "Record registrado";
        else msg ="No registr el record";
        Toast.makeText(this,  msg, Toast.LENGTH_LONG).show();
              
        miBD.close();
        
        accionBotones(false);
        
   
    }
    ///////////////////////////////////////////////////////
    private void accionBotones(Boolean bInd){
    	// se activan o desactivan los dos botones de respuesta
        // segn la lgica del juego
        Button oButtonSi = (Button) findViewById(R.id.Button_Si);
        oButtonSi.setEnabled(bInd);
        Button oButtonNo = (Button) findViewById(R.id.Button_No);
        oButtonNo.setEnabled(bInd);
        
        // si el indicador es false, se termina el juego
        // por lo que se se actualizan las preferencias
        // compartidas
        if (!bInd) {
        	// se actualiza la secuencia en las preferencias
        	// para inciar otro juego
            Editor editor = oDatosUsuario.edit();
            editor.putInt(PREFERENCIAS_SECUENCIA, 0);
            editor.putInt(PREFERENCIAS_PUNTOS, 0); 
            editor.putInt(PREFERENCIAS_FALLOS, 0);
            // se confirman los cambios en SharedPreferences
            editor.commit();
        }
    }
    
 
    ///////////////////////////////////////////////////////
    // leerArchivoPreguntas
    // limpia el objeto hashtable
    // obtiene el archivo XML con las preguntas
    // segn el idioma elegido
    // y se realiza un loop de lectura sobre el archivo XML
    // poor cada etiqueta <pregunta> se accede a sus atributos
    // para crear los campos del objeto Pregunta 
    // que se aade al objeto hashtable oListaPreguntas   
    ///////////////////////////////////////////////////////
    private void leerArchivoPreguntas(int intDesdePregunta) throws XmlPullParserException, IOException {
        
    	// Se limpia el objeto hashtable anterior
        oListaPreguntas.clear();

        // se define un objeto XmlResourceParser 
        XmlResourceParser preguntasXML;
        
        // selecciona el archivo XML que corresponda
        // segn el idioma elegido
        Integer intIdioma = oDatosUsuario.getInt(PREFERENCIAS_IDIOMA, 0);
        
        switch (intIdioma){
        	case 0: preguntasXML = getResources().getXml(R.xml.preguntas_cast); break;
        	case 1: preguntasXML = getResources().getXml(R.xml.preguntas_cat); break;
        	case 2: preguntasXML = getResources().getXml(R.xml.preguntas_eng); break;
        	default: preguntasXML = getResources().getXml(R.xml.preguntas_cast);
        }
        
        // Analiza el archivo XML 
        int eventoParse = -1;
        intPreguntasCargadas = 0;

        // Recorre el documento hasta llegar a su final
        while (eventoParse != XmlResourceParser.END_DOCUMENT) {
        	
        	// si se lee una etiqueta de inicio
            if (eventoParse == XmlResourceParser.START_TAG) {

                // Obtiene el nombre de la etiqueta
                String strTag = preguntasXML.getName();
                
                // Slo nos interesan las etiquetas "pregunta"
                // por si acaso, las filtramos
                if (strTag.equals(TAG_PREGUNTA)) {
                	
                	 String   preguntaStrSecuencia = preguntasXML.getAttributeValue(null, TAG_PREGUNTA_SECUENCIA);
                     Integer  preguntaSecuencia = new Integer(preguntaStrSecuencia);
                	// slo nos interesan las preguntas desde una
                	// secuencia determinada
                	if (preguntaSecuencia >= intDesdePregunta){
                		
                		// se obtienen los campos desde los atributos de la etiqueta "pregunta"
                		// para crear un objeto Pregunta en la hashtable 
                		String   preguntaTexto = preguntasXML.getAttributeValue(null, TAG_PREGUNTA_TEXTO);
                		String   preguntaImagen = preguntasXML.getAttributeValue(null, TAG_PREGUNTA_IMAGEN);
                		String   preguntaRespuesta = preguntasXML.getAttributeValue(null, TAG_PREGUNTA_RESPUESTA);
                		String   preguntaStrValor = preguntasXML.getAttributeValue(null, TAG_PREGUNTA_VALOR);
                		Integer  preguntaValor = new Integer(preguntaStrValor);

                		// Guarda los datos del nuevo objeto Pregunta en 
                		// la tabla hashtable oListaPreguntas
                		oListaPreguntas.put(preguntaSecuencia, new Pregunta(preguntaSecuencia, preguntaTexto, preguntaImagen, preguntaRespuesta, preguntaValor));
                	    intPreguntasCargadas = intPreguntasCargadas + 1; 
                	}
                }
            }
            // se mueve al siguiente elemento 
            eventoParse = preguntasXML.next();
        }
    }
  
	
    /////////////////////////////////////////////////////
	// Crea un  objeto ImageView para la pregunta
	// siguiente y que reemplazar
	// al objeto ImageView que se visualiza actualmente
	// para mejorar la transicin  	  
	/////////////////////////////////////////////////////
	private class CrearImageViewSiguiente implements ViewSwitcher.ViewFactory {
		 // el mtodo abstracto makeView()
		 // crea un objeto View que se agregar a un 
		 // objeto ViewSwitcher.
		public View makeView() {
			LayoutInflater oLInflater= LayoutInflater.from(getApplicationContext());
			ImageView oIView = (ImageView) oLInflater.inflate(R.layout.jugar_imageswitcher, oImagenPregunta, false);	
					
			return oIView;
		}
	}

	////////////////////////////////////////////////////
	// Crea un  objeto TextView para la pregunta
	// siguiente y que reemplazar
	// al objeto TextView que se visualiza actualmente
	// para mejorar la transicin  	  
	////////////////////////////////////////////////////
	private class CrearTextViewSiguiente implements ViewSwitcher.ViewFactory {
		// 	el mtodo abstracto makeView()
		// crea un objeto View que se agregar a un 
		// objeto ViewSwitcher.
		public View makeView() {
			LayoutInflater oLInflater= LayoutInflater.from(getApplicationContext());			
			TextView oTView = (TextView) oLInflater.inflate(R.layout.jugar_textswitcher, oTextoPregunta, false);
			return oTView;
		}
	}
	
    ///////////////////////////////////////////////////////
    // Objeto que representa a cada pregunta de la lista      
    ///////////////////////////////////////////////////////
    private class Pregunta {

    	Integer mSecuencia;
        String  mTexto;
        String  mImagen;
        String  mRespuesta;
        Integer mValor;
        
        // Constructor del objeto Pregunta
        public Pregunta(int preguntaSecuencia, 
        				String preguntaTexto, 
        				String preguntaImagen, 
        				String preguntaRespuesta, 
        				int preguntaValor) {
        	// asigna valores a las variables miembro 
        	mSecuencia = preguntaSecuencia;
            mTexto     = preguntaTexto;
            mImagen    = preguntaImagen;
            mRespuesta = preguntaRespuesta;
            mValor     = preguntaValor;            
        }
    }
    ////////////////////////////////////////////////////////
    // mtodos auxiliares para obtener los campos del objeto Pregunta
    //////////////////////////////////////////////////////////
    private String getTextoPregunta(Integer intSecuencia) {
        String texto = null;
        Pregunta oPreguntaActual = (Pregunta) oListaPreguntas.get(intSecuencia);
        if (oPreguntaActual != null) {
            texto = oPreguntaActual.mTexto;
        }
        return texto;
    }    
    private String getRespuestaPregunta(Integer intSecuencia) {
        String texto = null;
        Pregunta oPreguntaActual = (Pregunta) oListaPreguntas.get(intSecuencia);
        if (oPreguntaActual != null) {
            texto = oPreguntaActual.mRespuesta;
        }
        return texto;
    }
    private Integer getValorPregunta(Integer intSecuencia) {
        Integer valor = 0;
        Pregunta oPreguntaActual = (Pregunta) oListaPreguntas.get(intSecuencia);
        if (oPreguntaActual != null) {
            valor = oPreguntaActual.mValor;
        }
        return valor;
    }
    private Drawable getImagenPregunta(int intSecuencia) {
        Drawable imagen;

        try {
            // se obtiene el nombre de la imagen en asset
            String imgName = getImagenName(intSecuencia);
            
            // se abre el flujo con el archivo indicado           
            InputStream oIStream=getAssets().open(imgName,AssetManager.ACCESS_STREAMING);
             
            // se decodifica el flujo de datos 
            // en un objeto Bitmap
            Bitmap oBmp = BitmapFactory.decodeStream(oIStream);            
            imagen = new BitmapDrawable(getResources(), oBmp);
        } catch (Exception e) {
        	// ante un error, se muestra una imagen local           
            imagen = getResources().getDrawable(R.drawable.nohayimagen);
        }
        return imagen;
    }
    private String getImagenName(Integer intSecuencia) {
        String imagenName = null;
        Pregunta oPreguntaActual = (Pregunta) oListaPreguntas.get(intSecuencia);
        if (oPreguntaActual != null) {
            imagenName = oPreguntaActual.mImagen;
        }
        return imagenName;
    }
    
    
    
    //////////////////////////////////////////////////////
    @Override
    public boolean  onCreateOptionsMenu(Menu menu) {
    	// si se pulsa el botn MENU
    	
        super.onCreateOptionsMenu(menu);
        
        // se crea el men a partir del recurso menu
        getMenuInflater().inflate(R.menu.menu, menu);
        
        // a cada elemento del men se asocia un
        // objeto Intent
        menu.findItem(R.id.item1).setIntent(new Intent(this, UsuarioAct.class));
        menu.findItem(R.id.item2).setIntent(new Intent(this, ResultadosAct.class));
        menu.findItem(R.id.item3).setIntent(new Intent(this, AyudaAct.class));
        
        // el mtodo debe devolver un booleano
        return true;
    }

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// inicia la actividad solicitada
		
		super.onOptionsItemSelected(item);
		startActivity(item.getIntent());
		return true;
	}
	
	
	
	
}