package com.inforbooks.geociencia.p01;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;

import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;


public class UsuarioAct extends GeoCienciaAct {
	
	SharedPreferences oDatosUsuario;	

	/** Se llama al crearse el objeto */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.usuario);
		
		// Recupera las preferencias compartidas
		oDatosUsuario = getSharedPreferences(PREFERENCIAS, Context.MODE_PRIVATE);
		 
		// Recupera el nombre del usuario
		getNombreUsuario();
		
		// Recupera la contrasea
		getPassword();
		
		// Recupera el idioma
		getListaIdiomas();
		  
	}
	
	/**
	 * Obtiene el nombre del usuario
	 */	 
	private void getNombreUsuario() {
		// obtiene un objeto EditText para que contenga el nombre
		// del usuario
		EditText objEt = (EditText) findViewById(R.id.EditText_Nombre);
		
		if (oDatosUsuario.contains(PREFERENCIAS_NOMBRE)) {
			// asigna el nombre en el control o ""
			objEt.setText(oDatosUsuario.getString(PREFERENCIAS_NOMBRE, "xx"));
		}
	}
 
	/**
	 * Informa el estado de la contrasea
	 */	 
	private void getPassword() {
		// obtiene un objeto TextView para el 
		// estado de informacin
		// de la contrasea
		TextView objTv = (TextView) findViewById(R.id.TextView_Estado_Password);
		
		// Comprueba si hay contrasea informada
		if (oDatosUsuario.contains(PREFERENCIAS_PSW)) {
			// tiene contrasea registrada
			objTv.setText(R.string.usuario_psw_registrada);
		} else {
			// no tiene contrasea
			objTv.setText(R.string.usuario_sin_psw);
		}
	}
	
	/**
	 * Lista de idiomas 
	 */
	 
	private void getListaIdiomas() {
		// Obtiene el objeto Spinner		
		final Spinner spinner = (Spinner) findViewById(R.id.Spinner_Idioma);
		
		// crea un adaptador y  lo carga con el array
		ArrayAdapter<?> adapter = ArrayAdapter.createFromResource(this,
				R.array.idiomas, android.R.layout.simple_spinner_item);		
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		
		// asigna el adaptador al objeto Spinner
		spinner.setAdapter(adapter);
		
		// asigna la seleccin vigente
		if (oDatosUsuario.contains(PREFERENCIAS_IDIOMA)) {
			spinner.setSelection(oDatosUsuario.getInt(PREFERENCIAS_IDIOMA, 0));
		}
		
		// se crea una funcin que est atenta a las selecciones
		// en la lista
		// se detecta cuando haya las selecciones en la lista 
		spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			// si se ha seleccionado un elemento de la lista
			public void onItemSelected(AdapterView<?> parent,
					View itemSelected, int selectedItemPosition, long selectedId) {
				
				// se crea un editor para las preferencias
				Editor editor = oDatosUsuario.edit();
				
				// se asigna la seleccin realizada
				editor.putInt(PREFERENCIAS_IDIOMA, selectedItemPosition);
				
				// se confirma el cambio en las preferencias
				editor.commit();
			}

			public void onNothingSelected(AdapterView<?> parent) {
				// implementacin requerida (aunque sea vaca)
			}
		});
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		
		// se obtiene el objeto de edicin
		EditText objEt = (EditText) findViewById(R.id.EditText_Nombre);
        
		// se obtiene el texto del control 
		String strNombre = objEt.getText().toString();
		
		// se crea un editor para las preferencias		
		Editor editor = oDatosUsuario.edit();
		
		// se coloca el texto en las preferencias
		editor.putString(PREFERENCIAS_NOMBRE, strNombre);

		// se confirma el cambio
		editor.commit();
	}

	/**
	 * Si se hace clic en el botn de contrasea
	 * funcin definida en usuario.xml
	 * atributo android:onClick
	 */
	public void abrirDlgPassword(View view) {
		    // se inician los pasos de modificar o crear contrasea
			showDialog(0);
	}
	
	@Override
    protected Dialog onCreateDialog(int d) {  
		
            // se crea el objeto LayoutInflater
		    // crea una instancia del diseo xml
			// getSystemService(String) recupera una instancia de 
			// LayoutInflater que est conectada al contexto vigente
			// y configurada para el dispositivo en el que se ejecuta 
			// la aplicacin 
		    LayoutInflater objLi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		    
		    // se obtiene  el objeto View 
            final View objVlayout = objLi.inflate(R.layout.usuario_password, (ViewGroup) findViewById(R.id.linearlayout01));           
            
            
            final EditText objEt1 = (EditText) objVlayout.findViewById(R.id.EditText_Password);
            final EditText objEt2 = (EditText) objVlayout.findViewById(R.id.EditText_Confirma);
            final TextView objTv = (TextView) objVlayout.findViewById(R.id.TextView_Mensaje);
             
            // se aade un objeto TextWatcher a la lista 
            // de los mtodos llamados cada vez que cambia el texto
            // en el objeto TextView objEt2
            // el objeto TextWatcher est relacionado 
            // con el objeto editable
            // sus mtodos se llaman cuando cambia el texto. 
            objEt2.addTextChangedListener(new TextWatcher() {
                @Override
                public void afterTextChanged(Editable s) {
                	// se obtienen las dos cadenas
                	// password y confirmacin
                	// para verificar la igualdad
                    String strPassword = objEt1.getText().toString();
                    String strConfirma = objEt2.getText().toString();
                    
                    // se coloca el mensaje
                    if (strPassword.equals(strConfirma)) {
                        objTv.setText(R.string.usuario_psw_ok);
                    } else {
                        objTv.setText(R.string.usuario_psw_ko);
                    }
                }

                // mtodos que hay que definir aunque se dejen vacos
                @Override
                public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                }

                @Override
                public void onTextChanged(CharSequence s, int start, int before, int count) {
                }
            });
            
            AlertDialog.Builder objBuilder = new AlertDialog.Builder(this);
            objBuilder.setView(objVlayout);
            
            // se configura el objeto AlertDialog
            objBuilder.setTitle(R.string.usuario_btn_inpassword);
            
            // clic en el botn Cancel
            objBuilder.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int whichButton) {
                    // se elimina el dilogo
                    UsuarioAct.this.removeDialog(0);
                }
            });
            
            // se hace clic en Aceptar
            objBuilder.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int which) {
                    TextView objTvMensajeEstado = (TextView) findViewById(R.id.TextView_Estado_Password);
                    
                    // recupera el valor de los cuadros de edicin
                    String strPassword = objEt1.getText().toString();
                    String strConfirma = objEt2.getText().toString();
                    
                    // se verifica que haya coincidencia
                    if (strPassword.equals(strConfirma)) {
                        Editor editor = oDatosUsuario.edit();
                        
                        // se guarda la contrasea en las preferencias
                        editor.putString(PREFERENCIAS_PSW, strPassword);
                        editor.commit();
                        objTvMensajeEstado.setText(R.string.usuario_btn_inpassword);
                    } else {
                    	// se informa que no se cambia la contrasea
                    	objTvMensajeEstado.setText(R.string.usuario_msg_password_ko);                       
                    }
                 // se elimina el dilogo
                    UsuarioAct.this.removeDialog(0);
                }
            });
            // Crea el dilogo AlertDialog 
            AlertDialog objDialogPassword = objBuilder.create();
            return objDialogPassword;
        }	   
}