package com.inforbooks.intentdatos;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.Menu;
import android.view.View;
import android.widget.Toast;

public class MainActivity extends Activity {

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
    }

    public void onClick(View view) {
    	// Preparamos un objeto Intent para 
    	// invocar otra actividad
        Intent intento = new 
                Intent("com.inforbooks.intentdatos.OtraActividad");

        // Adjuntar datos al objeto Intent
        
        // Se utiliza el mtodo putExtra() 
        // para aadir datos de diferentes tipos
        // primitivos (String, Float, Int, etc.)
        // o un objeto Serializable
        // en la forma de clave/valor            
        intento.putExtra("1String", "Dato 1 de tipo string");
        intento.putExtra("2Float", 50.60f);
        intento.putExtra("3Int", 7);
        intento.putExtra("4Boolean", true);
        
        // Crea un nuevo objeto de la clase auxiliar
        // serializable
        UnaClaseAuxiliar objCualquiera = new UnaClaseAuxiliar();
        objCualquiera.setID(5);
        objCualquiera.setNombre("Salas, Antonio");
        objCualquiera.setPais("Portugal");
        intento.putExtra("ObjetoUnaClaseAuxiliar", objCualquiera);
        
        // Se usa un objeto Bundle para aadir  
        // conjunto de pares de valores extras
        // empaquetas dentro de un objeto Bundle
        Bundle extras = new Bundle();
        extras.putString("5String", "Cadena empaquetada en objeto Bundle");
        extras.putBoolean("6Boolean", false); 
        extras.putDouble("7Double", 300.50); 
        
        // Se adjunta el objeto Bundle 
        // al objeto Intent  
        intento.putExtras(extras);  
        
        // Inicia una actividad que devolver datos
        // el parmetro RequestCode con un valor 
        // >= 0 hace que se cree una subactivitidad
        // y esta actividad queda a la espera del resultado
        startActivityForResult(intento, 1);        
    }
    
    // Recibe los resultados de la actividad invocada     
    public void onActivityResult(int requestCode, 
            int resultCode, Intent data)
    {
        // Comprueva que el parmetro requestCode sea 1 
        if (requestCode == 1) {

            // Si el cdigo resultado es OK  
            if (resultCode == RESULT_OK) {

                // Obtiene el resultado con getData() 
                Uri url = data.getData();
                Toast.makeText(this, 
                		"Dato de retorno" +
                		"\nDireccin WEB : \n" + url.toString(), 
                        Toast.LENGTH_SHORT).show();
                
                // Obtiene los datos de retorno de 
            	// la invocacin a la actividad  
                Toast.makeText(this, "Dato de retorno" +
                		"\nVerdadero o Falso : \n" + 
                		Boolean.toString(data.getBooleanExtra("True/False", true)), 
                        Toast.LENGTH_SHORT).show();
                
                // Obtiene los datos de retorno de 
            	// la invocacin a la actividad  
                Toast.makeText(this, "Dato de retorno" +
                		"\nNmero recibido: \n" +	
                		Integer.toString(data.getIntExtra("Nro", 0)), 
                        Toast.LENGTH_SHORT).show();  

          
            }            
        }
    }
    @Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_main, menu);
		return true;
	}

}

	


