package com.inforbooks.lectorproveedorcontenidos;


import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

public class LectorProveedor extends ContentProvider  {
	
	 public static final Uri CONTENT_URI = 
		      Uri.parse("content://com.inforbooks.lectorproveedor/noticias");
		  
		  //Nombres de los campos de la base de datos
		  public static final String COL_ID = "_id";
		  public static final String COL_DATE = "date";
		  public static final String COL_TITLE = "title";
		  public static final String COL_DESCRIPTION = "description";		 
		  public static final String COL_CATEGORY = "category";
		  public static final String COL_AUTHOR = "author";
		  public static final String COL_LINK = "link";

		  
		  NoticiasDataBaseHelper dbHelper;

		  @Override
		  public boolean onCreate() {
		    Context context = getContext();
		    
		    // Crea la base de datos
		    // la base de datos se abrir cuadndo se haga
		    // el primer acceso 

		    dbHelper = new NoticiasDataBaseHelper(context,
		      NoticiasDataBaseHelper.DATABASE_NAME, null,
		      NoticiasDataBaseHelper.DATABASE_VERSION);

		    return true;
		  }
		  
		

		  // Constantes para los 2 tipos de consultas 
		  private static final int TODO = 1;
		  private static final int UNA_NOTICIA = 2;
		  

		  private static final UriMatcher uriMatcher;

		  // Asigna valores del objeto UriMatcher
		  // la direccin URI que termina en noticias
		  // extrae todas las noticias
		  // si se indica un identificador 
		  // extrae una noticia determinada  
		  static {
		   uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
		   uriMatcher.addURI("com.inforbooks.lectorproveedor", "noticias", TODO);
		   uriMatcher.addURI("com.inforbooks.lectorproveedor", "noticias/#", UNA_NOTICIA);
		  }
		  
		  @Override
		  public String getType(Uri uri) {
			  // Devuelve una cadena que indica
			  // el tipo MIME de la direccin URI
			  // del proveedor de contenidos
		    switch (uriMatcher.match(uri)) {
		      case TODO  : return "vnd.android.cursor.dir/vnd.inforbooks.noticias";
		      case UNA_NOTICIA: return "vnd.android.cursor.item/vnd.inforbooks.noticias";
		      default: throw new IllegalArgumentException("URI no prevista: " + uri);
		    }
		  }


		  @Override
		  public Cursor query(Uri uri,
		                      String[] columnas,
		                      String seleccion,
		                      String[] params,
		                      String orden) {

			  
		    SQLiteDatabase bdAux;
		    
		    try {
		    	bdAux =  dbHelper.getWritableDatabase();
		    } catch (SQLiteException ex){
		    	bdAux =  dbHelper.getReadableDatabase();
		    }
		   

		    // Se crea un objeto SQLiteQueryBuilder
		    // para realizar la consulta 
		    SQLiteQueryBuilder queryBuilder = new SQLiteQueryBuilder();		 

		    // Cuando se busca una fila determinada
		    // el resultado ser 0 o 1 fila.
		    switch (uriMatcher.match(uri)) {
		      case UNA_NOTICIA: 
		    	  queryBuilder.appendWhere(COL_ID + "=" + uri.getPathSegments().get(1));
		          break;
		      default: break;
		    }

		    // Se indica si hay clasificacin
		    // del conjunto de resultados
		    // (por defecto, se ordena por fecha y hora) 
		    String orderBy;
		    if (TextUtils.isEmpty(orden)) {
		      orderBy = COL_DATE;
		    } else {
		      orderBy = orden;
		    }

		    // Se indica el nombre de la tabla de la consulta
		    queryBuilder.setTables(NoticiasDataBaseHelper.TABLA_NOTICIAS);
		    
		    // Se ejecuta la consulta
		    Cursor cursor = queryBuilder.query(bdAux,
		                        columnas,
		                        seleccion, params,
		                        null, null,
		                        orderBy);

		    // Se registra para notificar los cambios
		    // del cursor 
		    cursor.setNotificationUri(getContext().getContentResolver(), uri);

		    // Devuelve un cursor con el resulset
		    return cursor;
		  }

		  @Override
		  public Uri insert(Uri _uri, ContentValues _initialValues) {
			// Abre la base de datos en modo write
			// ya que se va a modificar el contenido
		    SQLiteDatabase bDatos = dbHelper.getWritableDatabase();
		    
		    // Inserta una nueva fila;   
		    // el mtodo insert devuelve el ID cuando
		    // la insercin es correcta.
		    // Tema:nullColumnhack
		    // El segundo parmetro indica una columna que se asignar con NULL
		    // cuando _initialValues est vaco. Este truco es necesario ya que
		    // SQLite no permite una fila totalmente vaca
		    // (otros SQL s); por eso est este invento
		    // del 2do. parmetro nullColumnHack; se debe indicar cualquier
		    // columna de la tabla que acepte null.
		    // Si sabemos que esto nunca pasar, en el 2do parmetro
		    // se podra indicar null (en lugar de nombre de columna)
		    long rowID = bDatos.insert(
		      NoticiasDataBaseHelper.TABLA_NOTICIAS, "description", _initialValues);

		    // Obtener el Uri de la ltima fila insertada.
		    if (rowID > 0) {
		      Uri uri = ContentUris.withAppendedId(CONTENT_URI, rowID);
		      
		      // Se notifican a los observadores que
			  // hubo cambios en la base de datos
		      getContext().getContentResolver().notifyChange(uri, null);
		      
		      // Devuelve el uri de la fila nueva 
		      return uri;
		    }
		    
		    throw new SQLException("No puede aadir la fila en " + _uri);
		  }

		  @Override
		  public int delete(Uri uri, String where, String[] whereArgs) {
			  
			// Abre la base de datos en modo write
			// ya que se va a modificar el contenido
		    SQLiteDatabase bDatos = dbHelper.getWritableDatabase();
		    
		    // Variable para devolver la cantidad de filas afectadas
		    // por la eliminacin
		    int qFilas;
		    // Segn la direccin Uri se elige
		    // el tipo de eliminacin
		    switch (uriMatcher.match(uri)) {
		      case TODO:
		        qFilas = bDatos.delete(
		          NoticiasDataBaseHelper.TABLA_NOTICIAS, where, whereArgs);
		        break;
		      case UNA_NOTICIA:
		    	// obtiene el valor del identificador
		        String segment = uri.getPathSegments().get(1);
		        // Elimina una fila (si existe)
		        qFilas = bDatos.delete(NoticiasDataBaseHelper.TABLA_NOTICIAS, 
		                COL_ID + "=" 
		                + segment
		                + (!TextUtils.isEmpty(where) ? " AND ("
		                + where + ')' : ""), whereArgs);
		        break;

		      default: throw new IllegalArgumentException("URI no prevista: " + uri);
		    }

		    // Se notifican a los observadores que
		    // hubo cambios en la base de datos
		    getContext().getContentResolver().notifyChange(uri, null);
		    
		    // Devuelve la cantidad de filas eliminadas
		    return qFilas;
		  }

		  @Override
		  public int update(Uri uri, ContentValues values, 
		             String where, String[] whereArgs) {
			// Abre la base de datos en modo write
			// ya que se va a modificar el contenido
			SQLiteDatabase bDatos = dbHelper.getWritableDatabase();
		   
		    // Contador de filas afectadas por la modificacin
		    int qFilas;
		    switch (uriMatcher.match(uri)) {
		      case TODO: 
		        qFilas = bDatos.update(NoticiasDataBaseHelper.TABLA_NOTICIAS, 
		                                values, where, whereArgs);
		        break;
		      case UNA_NOTICIA: 
		        String segment = uri.getPathSegments().get(1);
		        qFilas = bDatos.update(NoticiasDataBaseHelper.TABLA_NOTICIAS, 
		                                values, COL_ID
		                                  + "=" + segment
		                                  + (!TextUtils.isEmpty(where) ? " AND ("
		                                  + where + ')' : ""), whereArgs);
		        break;
		      default: throw new IllegalArgumentException("URI no prevista: " + uri);
		    }

		    // Se notifican a los observadores que
		    // hubo cambios en la base de datos
		    getContext().getContentResolver().notifyChange(uri, null);
		    // Devuelve la cantidad de filas afectadas
		    return qFilas;
		  }
		  
		  // Clase auxiliar basada en SQLiteOpenHelper
		  // para crear la base de datos  
		  private static class NoticiasDataBaseHelper extends SQLiteOpenHelper {
		  
		    private static final String TAG = "LectorProveedor";
		  
		    private static final String DATABASE_NAME = "noticias.db";
		    private static final int DATABASE_VERSION = 1;
		    private static final String TABLA_NOTICIAS = "noticias";
		  
		    private static final String DATABASE_CREATE =
		      "create table " + TABLA_NOTICIAS + " ("
		      + COL_ID + " integer primary key autoincrement, "
		      + COL_DATE + " INTEGER, "		     
		      + COL_TITLE + " TEXT, "
		      + COL_DESCRIPTION + " TEXT, "
		      + COL_CATEGORY + " TEXT, "
		      + COL_AUTHOR + " TEXT, "
		      + COL_LINK + " TEXT);";
		  
		  
		    public NoticiasDataBaseHelper(Context context, String name,
		                                    CursorFactory factory, int version) {
		      super(context, name, factory, version);
		    }
		  
		    @Override
		    public void onCreate(SQLiteDatabase db) {
		      db.execSQL(DATABASE_CREATE);
		    }
		  
		    @Override
		    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		      Log.w(TAG, "Se actualiza la versin de la base de datos " + oldVersion + " a la versin "
		                 + newVersion + ", se eliminarn los datos.");
		   
		      db.execSQL("DROP TABLE IF EXISTS " + TABLA_NOTICIAS);
		      onCreate(db);
		    }
		  }
		}


