package com.inforbooks.lectorproveedorcontenidos;

import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.os.IBinder;
import android.os.SystemClock;
import android.util.Log;

public class ActualizadorService extends Service {

	public static String CLASE="ActualizadorService";
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}
	
	private void addNoticia(Noticia _noticia) {	

	    // Se cambia el modo de obtener el objeto ContentResolver   
	    ContentResolver cr = getContentResolver();
	    
	    // Crea un where para verificar que la fila no exista	
	    String where = LectorProveedor.COL_DATE + " = " + _noticia.getDate().getTime();

	    // Si es una noticia nueva, la inserta
	    Cursor query = cr.query(LectorProveedor.CONTENT_URI, null, where, null, null);
	    if (query.getCount()==0) {
	      ContentValues values = new ContentValues();

	      values.put(LectorProveedor.COL_DATE, _noticia.getDate().getTime());
	      values.put(LectorProveedor.COL_TITLE, _noticia.getTitle());
	      values.put(LectorProveedor.COL_DESCRIPTION, _noticia.getDescription());   
	      values.put(LectorProveedor.COL_CATEGORY, _noticia.getCategory());
	      values.put(LectorProveedor.COL_AUTHOR, _noticia.getAuthor());	 
	      values.put(LectorProveedor.COL_LINK, _noticia.getLink());

	      cr.insert(LectorProveedor.CONTENT_URI, values);
	    }
	    query.close();
	  }
	
	public void accederNoticias() {
		 
		
	    // Acceso al archivo XML
	    URL url;
	    try {
	      // En el recurso rss_news
	      // se define la cadena de acceso al recurso 
	      // por ejemplo, "http://www.xxx.com/feeds/rss.xml"	
	      String rssNews = getString(R.string.rss_news);
	      url = new URL(rssNews);

	      // Crea una conexin
	      URLConnection connection;
	      connection = url.openConnection();

	      HttpURLConnection httpConnection = (HttpURLConnection)connection;
	      int responseCode = httpConnection.getResponseCode();

	      if (responseCode == HttpURLConnection.HTTP_OK) {
	        InputStream in = httpConnection.getInputStream();

	        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	        DocumentBuilder db = dbf.newDocumentBuilder();

	        // Analiza el contenido
	        Document dom = db.parse(in);
	        Element docElem = dom.getDocumentElement();

	        // Obtiene los elementos de las noticias
	        NodeList nl = docElem.getElementsByTagName("item");
	        if (nl != null && nl.getLength() > 0) {
	          for (int i = 0 ; i < nl.getLength(); i++) {
	            Element entry = (Element) nl.item(i);
	            
	            // Contenido que nos interesan
	            String title= "";
	            String description="";
	            String category="";
	            String author = "";
	            String link ="";
	            
	            Element nodo_title = (Element) entry.getElementsByTagName("title").item(0);
	            if (nodo_title != null) {
	            title = nodo_title.getFirstChild().getNodeValue();}
	            
	            Element nodo_description = (Element) entry.getElementsByTagName("description").item(0);
	            if (nodo_description.getFirstChild() != null) {
	            description = nodo_description.getFirstChild().getNodeValue();}
	            
	            Element nodo_category = (Element) entry.getElementsByTagName("category").item(0);
	            if (nodo_category != null) {
	            category = nodo_category.getFirstChild().getNodeValue();}
	            
	            Element nodo_author = (Element) entry.getElementsByTagName("dc:creator").item(0);
	            if (nodo_author != null) {
	            author = nodo_author.getFirstChild().getNodeValue();}
	            
	            Element nodo_link = (Element) entry.getElementsByTagName("link").item(0);
	            if (nodo_link != null) {
	            link = nodo_link.getFirstChild().getNodeValue();}
	            
	            Element nodo_date = (Element) entry.getElementsByTagName("pubDate").item(0);
	            String fechaHora = nodo_date.getFirstChild().getNodeValue();
	            SimpleDateFormat sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.US);
	            Date date = new GregorianCalendar(0,0,0).getTime();
	            try {
	              date = sdf.parse(fechaHora);
	            } catch (ParseException e) {
	              Log.d(CLASE, "Excepcin en conversin de fecha hora.", e);
	            } 
	            
	            // Crea el objeto Noticia
	            final Noticia noticia = new Noticia(date, 
	            				title, 
	            				description, 
	            				category, 
	            				author, 
	            				link);

	            // Y se aade la noticia a la lista...
	            addNoticia(noticia);	            
	          }
	        }
	      }
	    } catch (MalformedURLException e) {
		  Log.d(CLASE, "MalformedURLException", e);
		} catch (Exception e) {
	      Log.d(CLASE, "Excepcin no especfica", e);
	    }
	    finally {
	    	// detiene el servicio que se 
	    	// despertar por la alarma
	    	stopSelf();
	    }
	  }
	
	
	@Override
	  public int onStartCommand(Intent intent, int flags, int startId) {
	    // Se obtiene los minutos de la frecuencia de 
	    // actualizacin (configurado por el usuario)
	    int actualizarCada = LectorNoticias.minutosFrecuencia * 60000;

	   
	    // si hay actualizacin automtica
	    // cada x milisegundos
	    if (LectorNoticias.actualizaAutomatica) {
	    	// Se define una alarma para que el servicio se despierte
	    	// cada un cierto tiempo y haga lo que tenga que hacer
	    	// se elige un tipo de alarma que despierta contando
	    	// incluso el tiempo en el que el dispositivo est 
	    	// en reposo
	    	int tipoAlarma = AlarmManager.ELAPSED_REALTIME_WAKEUP;
	    	
	    	// Calcula la siguiente hora de actualizacin
	    	// sumando la frecuencia de actualizacin 
	    	// configurada por el usuario
	    	long horaActualizar = SystemClock.elapsedRealtime() + actualizarCada;
	    	
	    	// Se elige el mtodo de repeticin inexacta 
	    	// ya que es ms eficiente en trminos
	    	// de ahorro de energa aunque
	    	// los ciclos no sern totalmente
	    	// exactos; en este caso, no es relevante eso
	    	alarmManager.setInexactRepeating(tipoAlarma, horaActualizar, actualizarCada, pendingIntent);	    	
	    	
	    }
	    else {
	    	// No hay actualizacin automtica
	    	// se cancela la alarma
	    	alarmManager.cancel(pendingIntent);
	    	
	    	// Se ordena una actualizacin fija
	    	Thread t = new Thread(new Runnable() {
	    		public void run() {
	    			accederNoticias(); 
	        }
	      });
	      t.start();
	    }

	    // Este es el modo ms adecuado para los
	    // servicios (requiere que usemos stopSelf()
	    return Service.START_NOT_STICKY;
	  };

	 
	  // Definicin de los mecanismos de alarma
	  private AlarmManager alarmManager;
	  private PendingIntent pendingIntent;
	  
	  @Override
	  public void onCreate() {
		  super.onCreate();
		  alarmManager = (AlarmManager)getSystemService(Context.ALARM_SERVICE);
		  
		  String accion = AlarmaNoticias.ACTION_ALARMA_NOTICIAS;
		  Intent intento = new Intent(accion);
		  
		  // Se define el intento pendiente
		  // el 2do. parmtero no se utiliza actualmente
		  // el 3er. parmetro describe el intento
		  // el 4to. parmetro es un indicador de cmo
		  // se utiliza el intento
		  pendingIntent = PendingIntent.getService(this, 0, intento, 0);
	  }
	 

}
