package com.inforbooks.lectorproveedorcontenidos;
import android.app.ListActivity;
import android.app.LoaderManager;
import android.app.SearchManager;
import android.content.ContentUris;
import android.content.CursorLoader;
import android.content.Intent;
import android.content.Loader;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;



public class BuscadorNoticias extends ListActivity implements 
LoaderManager.LoaderCallbacks<Cursor> {
  
  private SimpleCursorAdapter adapter;
  
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
      
    // Crea un adaptador  
    // se vincula a una lista
    adapter = new SimpleCursorAdapter(this,
      android.R.layout.simple_list_item_1, null,
      new String[] { LectorProveedor.COL_TITLE },
      new int[] { android.R.id.text1 }, 0);
    setListAdapter(adapter);

    // Inicializa el cursor
    getLoaderManager().initLoader(0, null, this);

    // Analiza el intento
    analizarIntento(getIntent());
  }
    
  @Override
  protected void onNewIntent(Intent intent) {
    super.onNewIntent(intent);
    // Analizar el nuevo intento
    // para verificar si se trata de una
    // accin de bsqueda
    analizarIntento(getIntent());
  }

  private static String CONSULTA_BUSCADOR = "BUSCAR";

  private void analizarIntento(Intent intent) {
    // Si la actividad tiene relacin con una bsqueda
	// obtendremos los resultados del cursor
    // en caso contrario, no se hace nada
    if (Intent.ACTION_SEARCH.equals(intent.getAction())) {
      String buscar = intent.getStringExtra(SearchManager.QUERY);

      // Se crean los parmetros para realizar la bsqueda
      // y se pasarn al objeto Loader 
      Bundle params = new Bundle();
      params.putString(CONSULTA_BUSCADOR, buscar);
      
      // Se reinicia el Loader para 
      // ejecutar una nueva consulta
      getLoaderManager().restartLoader(0, params, this);
    }
  }

  public Loader<Cursor> onCreateLoader(int id, Bundle params) {
    String buscar =  " ";
      
    if (params != null) {
      // Obtenemos el texto que se quiere buscar
      buscar = params.getString(CONSULTA_BUSCADOR);
    }

    // Armado de la sentencia SQL
    String[] columnas = { LectorProveedor.COL_ID, 
    						LectorProveedor.COL_TITLE};
    // Clusula where
    // en la variable buscar est el texto introducido 
    // en el cuadro de bsqueda
    String where = LectorProveedor.COL_TITLE
                     + " LIKE \"%" + buscar + "%\"";
    // Podra haber argumentos de reemplazo 
    // en este caso, es null
    String[] whereArgs = null;
    
    // Clusula Order By
    String orderBy = LectorProveedor.COL_TITLE + " ASC";
    
    // Crea el objeto Loader<Cursor>
    return new CursorLoader(this, LectorProveedor.CONTENT_URI,
            columnas, where, whereArgs,
            orderBy);
  }

  public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
    // Al finalizar la carga del cursor
	// se reemplaza el cursor en el adaptador.
    adapter.swapCursor(cursor);
  }

  public void onLoaderReset(Loader<Cursor> loader) {
    // Se elimina el contenido actual del adaptador
    adapter.swapCursor(null);
  }
  
  // Responder al click en la lista resultante  
  // en muchos casos nos interesa prolongar la accin
  // de bsqueda con alguna funcionalidad adicional
  // que se har sobre el elemento seleccionado
  // en la lista resultante
  @Override
  protected void onListItemClick(ListView listView, View view, int position, long id) {
    super.onListItemClick(listView, view, position, id);
    
    // Crea el Uri del elemento clicado
    Uri uriClicado = 
      ContentUris.withAppendedId(LectorProveedor.CONTENT_URI, id);
    
    // Crea un intento con 
    // ACTION_VIEW para visualizar
    // el elemento de la lista que ha recibido el clic.
    Intent intent = new Intent(Intent.ACTION_VIEW);
    intent.setData(uriClicado);
    
    // Inicia la actividad 
    // para visualizar el elemento
    startActivity(intent);
  }

}
