package com.inforbooks.lectorproveedorcontenidos;


import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.Spinner;


public class SettingsActivity extends Activity {


	  CheckBox chkAutomatica;
	  Spinner spinnerFrecuencia;
	  Spinner spinnerRango;

	  //Identificadores de los campos SharedPreferences
	  public static final String PREF_ACT_AUTOMATICA = "PREF_ACT_AUTOMATICA";
	  public static final String PREF_IDX_FRECUENCIA = "PREF_IDX_FRECUENCIA";
	  public static final String PREF_IDX_RANGO = "PREF_IDX_RANGO";

	  // Objeto SharedPreferences
	  SharedPreferences misPref;

	  @Override
	  public void onCreate(Bundle savedInstanceState) {
	    super.onCreate(savedInstanceState);
	    setContentView(R.layout.settings);
	    
	    // Obtiene referencias a los objetos de la UI
	    spinnerFrecuencia = (Spinner)findViewById(R.id.spinner_frecuencia);
	    spinnerRango = (Spinner)findViewById(R.id.spinner_rango);
	    chkAutomatica = (CheckBox)findViewById(R.id.chk_act_automatica);

	    // Frecuencia de actualizacin
	    cargarListaValoresFrecuencia();
	    
	    // rango de das de noticias
	    cargarListaValoresRango();

	    Context context = getApplicationContext();
	    misPref = PreferenceManager.getDefaultSharedPreferences(context);
	    actualizarControlesUIConfiguracion();

	    // Se crea un gestor del clic del botn Cancel	   
	    Button cancelButton = (Button) findViewById(R.id.cancelButton);
	    cancelButton.setOnClickListener(new View.OnClickListener() {

	      public void onClick(View view) {
	    	  SettingsActivity.this.setResult(RESULT_CANCELED);
	        finish();
	      }
	    });
	    
	    // Se crea un gestor del clic del botn OK
	    Button okButton = (Button) findViewById(R.id.okButton);
	    okButton.setOnClickListener(new View.OnClickListener() {

	      public void onClick(View view) {
	        guardarConfiguracion();
	        SettingsActivity.this.setResult(RESULT_OK);
	        finish();
	      }
	    });

	  } 

	private void guardarConfiguracion() {
		// Se han modificado las condiciones
		// de configuracin
	    int nuevoIdx1 = spinnerFrecuencia.getSelectedItemPosition();
	    int nuevoIdx2 = spinnerRango.getSelectedItemPosition();

	    boolean nuevoChk = chkAutomatica.isChecked();

	    // Usa el editor para cambiar los
	    // valores que se guardan en preferencias
	    Editor editor = misPref.edit();
	    editor.putBoolean(PREF_ACT_AUTOMATICA, nuevoChk);
	    editor.putInt(PREF_IDX_FRECUENCIA, nuevoIdx1);
	    editor.putInt(PREF_IDX_RANGO, nuevoIdx2);
	    // Confirma los cambios
	    editor.commit();
	  }


	private void actualizarControlesUIConfiguracion() {
		  
		// Actualiza checkbox de actualizacin
		// automtica
		// de manera predeterminada es false
	    boolean chk = misPref.getBoolean(PREF_ACT_AUTOMATICA, false);
	    chkAutomatica.setChecked(chk);
	    
	    // Actualiza la frecuencia elegida
	    // de manera predeterminada es 5 minutos
	    int idx1 = misPref.getInt(PREF_IDX_FRECUENCIA, 1);
	    spinnerFrecuencia.setSelection(idx1);
	    
	    // Actualiza el rango de noticias
	    // a 1 da
	    int idx2 = misPref.getInt(PREF_IDX_RANGO, 1);
	    spinnerFrecuencia.setSelection(idx2);

	   
	  }

	  private void cargarListaValoresFrecuencia() { 
	    // Carga la lista de frecuencias posibles
	    ArrayAdapter<CharSequence> aAdapter;
	    
	    // Crea el adaptador desde la matriz de textos de
	    // frecuencias
	    aAdapter = ArrayAdapter.createFromResource(this, R.array.txt_frecuencias,
	                                             android.R.layout.simple_spinner_item);
	    
	    int spinner_item = android.R.layout.simple_spinner_dropdown_item;
	    aAdapter.setDropDownViewResource(spinner_item);
	    
	    spinnerFrecuencia.setAdapter(aAdapter);

	  }
	  
	  private void cargarListaValoresRango() { 
		    // Carga la lista de rangos de noticias posibles
		    ArrayAdapter<CharSequence> aAdapter;
		    
		    // Crea el adaptador desde la matriz de textos de
		    // rangos
		    aAdapter = ArrayAdapter.createFromResource(this, R.array.txt_rangos,
		                                             android.R.layout.simple_spinner_item);
		    
		    int spinner_item = android.R.layout.simple_spinner_dropdown_item;
		    aAdapter.setDropDownViewResource(spinner_item);
		    
		    spinnerRango.setAdapter(aAdapter);

		  }

	}
