package com.inforbooks.lectorxmlsettings;

import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


import android.app.ListFragment;
import java.text.ParseException;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.widget.ArrayAdapter;


public class NewsListFragment extends ListFragment {

	 ArrayAdapter<Noticia> adaptador;
	 ArrayList<Noticia> noticias = new ArrayList<Noticia>();	  

	  @Override
	  public void onActivityCreated(Bundle savedInstanceState) {
	    super.onActivityCreated(savedInstanceState);

	    int layoutID = android.R.layout.simple_list_item_1;
	    
	    // Crea el adaptador con la lista noticias
	    adaptador = new ArrayAdapter<Noticia>(getActivity(), layoutID , noticias);
	    
	    setListAdapter(adaptador);
	    
	    // Crea un subproceso independiente
	    // comprobar las noticias
	    // desde la API 11 es obligatorio
	    // que las operaciones de red se 
	    // ejecuten en un subproceso
	    // de segundo plano (background)
	    Thread t = new Thread(new Runnable() {
	    	public void run() {
	    		accederNoticias();	 
	    	}
	    });
	    t.start();
	    
	  }
	  
	  
	  private Handler handler = new Handler();
	
	  private void accederNoticias() {
	    // Acceso al archivo XML
	    URL url;
	    try {
	      // En el recurso rss_news
	      // se define la cadena de acceso al recurso 
	      // por ejmplo, "http://www.xxx.com/feeds/rss.xml"	
	      String rssNews = getString(R.string.rss_news);
	      url = new URL(rssNews);

	      // Crea una conexin
	      URLConnection connection;
	      connection = url.openConnection();

	      HttpURLConnection httpConnection = (HttpURLConnection)connection;
	      int responseCode = httpConnection.getResponseCode();

	      if (responseCode == HttpURLConnection.HTTP_OK) {
	        InputStream in = httpConnection.getInputStream();

	        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	        DocumentBuilder db = dbf.newDocumentBuilder();

	        // Analiza el contenido
	        Document dom = db.parse(in);
	        Element docElem = dom.getDocumentElement();

	        // Limpia el array de noticias 
	        noticias.clear();

	        // Obtiene los elementos de las noticias
	        NodeList nl = docElem.getElementsByTagName("item");
	        if (nl != null && nl.getLength() > 0) {
	          for (int i = 0 ; i < nl.getLength(); i++) {
	            Element entry = (Element) nl.item(i);
	            
	            // Contenido que nos interesan
	            String title= "";
	            String description="";
	            String category="";
	            String author = "";
	            String link ="";
	            
	            Element nodo_title = (Element) entry.getElementsByTagName("title").item(0);
	            if (nodo_title != null) {
	            title = nodo_title.getFirstChild().getNodeValue();}
	            
	            Element nodo_description = (Element) entry.getElementsByTagName("description").item(0);
	            if (nodo_description.getFirstChild() != null) {
	            description = nodo_description.getFirstChild().getNodeValue();}
	            
	            Element nodo_category = (Element) entry.getElementsByTagName("category").item(0);
	            if (nodo_category != null) {
	            category = nodo_category.getFirstChild().getNodeValue();}
	            
	            Element nodo_author = (Element) entry.getElementsByTagName("dc:creator").item(0);
	            if (nodo_author != null) {
	            author = nodo_author.getFirstChild().getNodeValue();}
	            
	            Element nodo_link = (Element) entry.getElementsByTagName("link").item(0);
	            if (nodo_link != null) {
	            link = nodo_link.getFirstChild().getNodeValue();}
	            
	            Element nodo_date = (Element) entry.getElementsByTagName("pubDate").item(0);
	            String fechaHora = nodo_date.getFirstChild().getNodeValue();
	            SimpleDateFormat sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.US);
	            Date date = new GregorianCalendar(0,0,0).getTime();
	            try {
	              date = sdf.parse(fechaHora);
	            } catch (ParseException e) {
	              Log.d("NewsListFragment", "Excepcin en conversin de fecha hora.", e);
	            } 
	            
	            // Crea el objeto Noticia
	            final Noticia noticia = new Noticia(date, 
	            				title, 
	            				description, 
	            				category, 
	            				author, 
	            				link);

	            // Y se aade la noticia a la lista...
	            // El objeto Handler nos permite
	            // aadir un mensaje o un ejecutable
	            // en la lista de mensaje del mismo
	            // subproceso que lo invoca.	             
	            handler.post(new Runnable() {
	            	public void run() {
	            		 addNoticia(noticia);
	            	}
	            }); 
	            
	          }
	        }
	      }
	    } catch (MalformedURLException e) {
		  Log.d("NewsListFragment", "MalformedURLException", e);
		} catch (Exception e) {
	      Log.d("NewsListFragment", "Excepcin no especfica", e);
	    }
	    finally {
	    }
	  }

	  private void addNoticia(Noticia _noticia) {
	    // Aade la noticia a la lista
		noticias.add(_noticia);

	    // Notifica al adaptador que hubo cambio de datos.
	    adaptador.notifyDataSetChanged();
	  }
	 

	}
