package com.inforbooks.mapsgps;

import java.io.IOException;
import java.text.NumberFormat;
import java.util.List;
import java.util.Locale;


import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.Menu;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay; 

// se deriva desde la clase MapActivity

public class  GPSActivity extends MapActivity {
	MapView oMapView;
	MapController oMapController;
	GeoPoint oGeoPoint;
	MapOverlay oMapOverlay;	 
	
	private LocationManager oLManager;
	private LocationListener oLListener;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_gps);
        
        // se obtiene el objeto MapView
        // para utilizar sus mtodos
        // que permiten la configuracin del mapa
        oMapView = (MapView) findViewById(R.id.mapView);
        
        // opciones iniciales
        oMapView.setBuiltInZoomControls(true);
        oMapView.setSatellite(false);
        oMapView.setTraffic(false);
        
              
        // indicamos una posicin inicial para
        // la visin del mapa   
        double latitud = 50.0;
        double longitud = 10.0;
        
        // multiplicamos por 1.000.000
        oGeoPoint = new GeoPoint(
        		(int) (latitud * 1E6), 
        		(int) (longitud * 1E6));
        
        // se obtiene un controlador de mapas desde
        // la instancia del objeto MapView
        oMapController= oMapView.getController();
        
        // posicionar el mapa
        oMapController.animateTo(oGeoPoint);
        
        // establecer un nivel de zoom medio
        oMapController.setZoom(8); 
        
        // se fuerza el redibujo de la pantalla
        oMapView.invalidate();  
        
        // se crea el Overlay del mapa para detectar
        // los eventos de Touch
        MapOverlay oMapOverlay = new MapOverlay();
        List<Overlay> listOverlays = oMapView.getOverlays();
        listOverlays.clear();
        listOverlays.add(oMapOverlay);       
             
        // GPS
        // se obtiene el servicio de posicionamiento
        oLManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        
        // se crea una clase para detectar cambios de posicionamiento
        oLListener = new DetectarPosiciones();
        
        // se define cmo se actualizarn los cambios de posicionamiento
        oLManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,0, 0, oLListener);
        
        // deteccin de eventos
        
        // atender clics en checkbox 1
        CheckBox oCheckBoxSatelite = (CheckBox) findViewById(R.id.checkBox1);    
       
        oCheckBoxSatelite.setOnClickListener(new View.OnClickListener() {
    		@Override
    		public void onClick(View v) {
    			if(((CheckBox)v).isChecked())  
        			oMapView.setSatellite(true);
        		else
        			oMapView.setSatellite(false);    			
    		}
        }); 
        
        // atender clics en checkbox 2
        CheckBox oCheckBoxTrafico = (CheckBox) findViewById(R.id.checkBox2);    
       
        oCheckBoxTrafico.setOnClickListener(new View.OnClickListener() {
    		@Override
    		public void onClick(View v) {
    			if(((CheckBox)v).isChecked())  
        			oMapView.setTraffic(true);
        		else
        			oMapView.setTraffic(false);    			
    		}
        }); 
        
        // atender clics en checkbox 3
        CheckBox oCheckBoxZoom = (CheckBox) findViewById(R.id.checkBox3);    
       
        oCheckBoxZoom.setOnClickListener(new View.OnClickListener() {
    		@Override
    		public void onClick(View v) {    			
    			if(((CheckBox)v).isChecked())  
        			oMapView.setBuiltInZoomControls(true);
        		else
        			oMapView.setBuiltInZoomControls(false);			
    		}
        }); 
        
        // atender clics en botn IR GPS
        Button oButtonIr = (Button) findViewById(R.id.button1);    
       
        oButtonIr.setOnClickListener(new View.OnClickListener() {
    		@Override
    		public void onClick(View v) {
    			oMapController.animateTo(oGeoPoint);
        		oMapController.setZoom(20);
        		oMapView.invalidate();	
    		}
        }); 
    }
    
    private String buscarInfoAdicional(GeoPoint punto) {              
       
    	// Si no hay informacin
    	// se mostrara este texto
        String info = "No hay otra informacin.";
          
        if (punto != null) {
          double latitud = punto.getLatitudeE6() / 1E6;
          double longitud = punto.getLongitudeE6() / 1E6;
       
          // Se crea una instancia del objeto GeoCoder
          // para acceder al servicio de geocodificacin 
          Geocoder geocoder = new Geocoder(this, Locale.getDefault());

          try {
        	  // Como mximo se recogen 2 
        	  // informaciones para el punto
        	  List<Address> direcciones = geocoder.getFromLocation(latitud, longitud, 2);
        	  StringBuilder texto = new StringBuilder();
        	  // Si hay info se crea el texto
        	  if (direcciones.size() > 0) {
        		  Address direccion = direcciones.get(0);

        		  // Puede haber varias lneas
        		  // de direcciones
        		  for (int i = 0; i < direccion.getMaxAddressLineIndex(); i++)
        			  texto.append(direccion.getAddressLine(i)).append("\n");
        		  
        		  // Aade la localidad y el nombre del pas
        		  texto.append(direccion.getLocality()).append("\n");
        		  texto.append(direccion.getCountryName());
            }
            info = texto.toString();
          } catch (IOException e) {}
        }
        // Devuelve la informacin obtenida
        return info;  

      }
    
    
    private class DetectarPosiciones implements LocationListener{
    	@Override
    	public void onLocationChanged(Location loc) {
    		if (loc != null) {
    			TextView oTViewGps = (TextView) findViewById(R.id.textView1);    			 
    			
    			// formato con slo tres decimales
    			NumberFormat form = NumberFormat.getInstance();
    			form.setMaximumFractionDigits(4);
    			
    			oTViewGps.setText("GPS: Lat: " + form.format(loc.getLatitude()) +
    					" Lng: " + form.format(loc.getLongitude())); 
    			
    			// guardamos la posicin por
    			// si hay que navegar con el mapa
    			oGeoPoint = new GeoPoint(
        				(int) (loc.getLatitude() * 1E6),
        				(int) (loc.getLongitude() * 1E6));
    			
    			
    		} 
    	}
    	
    	@Override
    	public void onProviderDisabled(String provider){
    		
    	}
    	@Override
    	public void onProviderEnabled(String provider){
    		
    	}
    	@Override
    	public void onStatusChanged(String provider, int status, Bundle extras){
    		
    	}    	
    }
    class MapOverlay extends com.google.android.maps.Overlay {
    	 
    	
    	@Override
    	public boolean onTouchEvent(MotionEvent event, MapView mapView) {
    		// se produjo un toque en la pantalla
    		// y se informan las coordenadas 
    		// en la forma latitud-longitud
    		if (event.getAction()== 1){
    			GeoPoint punto = mapView.getProjection().fromPixels((int) event.getX(), (int) event.getY());
    			
    			// formato con slo 4 decimales
    			NumberFormat form = NumberFormat.getInstance();
    			form.setMaximumFractionDigits(4);
    			
    			// Obtener informacin adicional 
    			// del punto
    			String info = buscarInfoAdicional(punto);    			
    			
    			// se informa el punto GPS del clic
    			// o touch de la pantalla
    			Toast.makeText(getBaseContext(), "Ubicacin: " +
    					form.format(punto.getLatitudeE6() / 1E6) + " ," +
    					form.format(punto.getLongitudeE6()/1E6) + 
    					"\nInformacin sobre el punto:\n" + info,
    					Toast.LENGTH_LONG).show();
    		}
    		return false;
    	}
    }
        
    @Override
    protected boolean isRouteDisplayed() {
    	return false;
    } 
      
    
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.g, menu);
		return true;
	}

}
