package com.inforbooks.meteo;

import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.widget.TextView;

public class Meteo extends Activity {
  
	// Define las instancia privadas 
	// de los objetos de la clase
	private SensorManager sensorManager;
	private TextView sensorTermometro;
	private TextView sensorBarometro;
	private TextView sensorHigrometro;

	// Variables para almacenar los valores ledos
	private float valorTemp = Float.NaN;
	private float valorPres = Float.NaN;
	private float valorHum = Float.NaN;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
				
		// Relaciona elementos de la interfaz con las instancias de 
		// las instancias de los objetos 
		sensorTermometro = (TextView)findViewById(R.id.temperatura);
		sensorBarometro = (TextView)findViewById(R.id.presion);
		sensorHigrometro = (TextView)findViewById(R.id.humedad);
		
		// Obtener una instancia del SensorManager
		sensorManager = (SensorManager)getSystemService(Context.SENSOR_SERVICE);
		 
    
		// Creamos un temporizador para actualizar la
		// interfaz cada minuto
		Timer temporizador = new Timer("meteoTimer");
		
		// Define una tarea
		TimerTask tarea = new TimerTask(){
			public void run() {
				actualizarPantalla();
			}};
		
	    // Ejecuta la actualizacin cada minuto
	    // se utiliza un mtodo para que la
	    // accin se repita n veces
		// Planifica la repeticin de la tarea cada minuto 
		temporizador.scheduleAtFixedRate(tarea, 0, 60000);    	
  }


  // Implementacin de los escuchadores
  // de los sensores
  private final SensorEventListener escucharTemperatura = new SensorEventListener() {
  public void onAccuracyChanged(Sensor sensor, int accuracy) { }
  
    public void onSensorChanged(SensorEvent event) {
      valorTemp = event.values[0];
    }
  };
  
  private final SensorEventListener escuchadorPresion = new SensorEventListener() {
  
    public void onAccuracyChanged(Sensor sensor, int accuracy) { }
  
    public void onSensorChanged(SensorEvent event) {
      valorPres = event.values[0];
    }
  };
  
  private final SensorEventListener escuchadorHumedad = new SensorEventListener() {
  
    public void onAccuracyChanged(Sensor sensor, int accuracy) { }
  
    public void onSensorChanged(SensorEvent event) {
      valorHum = event.values[0];
    }
  };

  // Al reiniciarse la actividad
  // se vuelven a registrar los
  // escuchadores de los sensores
  @Override
  protected void onResume() {
    super.onResume();
    
    // Sensor de temperatura
    Sensor sensorTemperatura = 	sensorManager.getDefaultSensor(Sensor.TYPE_AMBIENT_TEMPERATURE);
	if (sensorTemperatura != null)
		sensorManager.registerListener(escucharTemperatura,
				sensorTemperatura,                                 
				SensorManager.SENSOR_DELAY_NORMAL);
	else
		sensorTermometro.setText("No se dispone de termmetro.");
    
	// Sensor de humedad relativa
    Sensor sensorHumedad = sensorManager.getDefaultSensor(Sensor.TYPE_RELATIVE_HUMIDITY);
    if (sensorHumedad != null)
      sensorManager.registerListener(escuchadorHumedad,
          sensorHumedad,                                 
          SensorManager.SENSOR_DELAY_NORMAL);
    else
      sensorHigrometro.setText("No se dispone de higrmetro.");
    
    // Sensor de presin atmosfrica
    Sensor sensorPresion = sensorManager.getDefaultSensor(Sensor.TYPE_PRESSURE);
    if (sensorPresion != null)	
      sensorManager.registerListener(escuchadorPresion,
          sensorPresion,                                 
          SensorManager.SENSOR_DELAY_NORMAL);
    else
      sensorBarometro.setText("No se dispone de barmetro.");  
   
  }

  // Este cdigo se ejecuta dentro del mismo subproceso de
  // la interfaz de usuario
  private void actualizarPantalla() {
    runOnUiThread(new Runnable() {
      public void run() {
    	  // Comprueba que el valor de la temperatura
    	  // sea un nmero vlido
    	  if (!Float.isNaN(valorTemp)) {
              sensorTermometro.setText(valorTemp + "C");
              sensorTermometro.invalidate();
          }
    	  // Comprueba que el valor de la presin
    	  // sea un nmero vlido
    	  if (!Float.isNaN(valorPres)) {
    		  sensorBarometro.setText(valorPres + "mm");
    		  sensorBarometro.invalidate();
    	  }
    	  // Comprueba que el valor de la humedad
    	  // sea un nmero vlido
    	  if (!Float.isNaN(valorHum)) {
    		  sensorHigrometro.setText(valorHum + "%");
    		  sensorHigrometro.invalidate();
    	  }
       
      }
    });
  };
  
  @Override
  protected void onPause() {
	// En cada pausa se anula el registro de los 
	// escuchadores
    sensorManager.unregisterListener(escuchadorPresion);
    sensorManager.unregisterListener(escucharTemperatura);
    sensorManager.unregisterListener(escuchadorHumedad);
    super.onPause();
  }
}