package com.inforbooks.meteo;

import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.text.format.Time;
import android.widget.TextView;

public class Orientacion extends Activity {
  
	// Define las instancia privadas 
	// de los objetos de la clase
	private static SensorManager sensorManager;
	
	// objetos de la interfaz
	private TextView azimuth;
	private TextView pitch;
	private TextView roll;
	private TextView observaciones;

	// Indica que hay lecturas correctas
	// en los sensores
	private static boolean sensor1 = false;
	private static boolean sensor2 = false;
	
	// Variables para almacenar los valores ledos
	private static float[] valoresAcelerometro;
	private static float[] valoresCampoMagnetico;
	
	// array para almacenar la ltima orientacin
	private static float [] values = new float[3];

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
				
		// Relaciona elementos de la interfaz con las instancias de 
		// las instancias de los objetos 
		azimuth = (TextView)findViewById(R.id.valorazimuth);
		pitch = (TextView)findViewById(R.id.valorpitch);
		roll = (TextView)findViewById(R.id.valorroll);
		observaciones = (TextView)findViewById(R.id.observaciones);
		
		// Obtener una instancia del SensorManager
		sensorManager = (SensorManager)getSystemService(Context.SENSOR_SERVICE);
				 
		// Comprobar que existen los sensores
		// y registrarlos
		if (comprobarSensores()){ 	
		 
			// Creamos un temporizador para actualizar la
			// interfaz cada minuto
			Timer temporizador = new Timer("orientTimer");
		
			// Define una tarea que se
			// ejecutar peridicamente
			// para actualiozar los valores
			// en pantalla
			TimerTask tarea = new TimerTask(){
					public void run() {
						actualizarPantalla();
					}
			};
		
			// Ejecuta la actualizacin cada minuto
			// se utiliza un mtodo para que la
			// accin se repita n veces
			// Planifica la repeticin de la tarea 
			// cada 1 segundo    
			temporizador.scheduleAtFixedRate(tarea, 0, 1000); 
		}
		else 
			observaciones.setText("No se dispone de los sensores necesarios.");
  }

	private boolean comprobarSensores() {
		  // Sensor ACELERMETRO
		    Sensor sensorAcelerometro = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
			if (sensorAcelerometro != null)
				sensorManager.registerListener(escuchadorAcelerometro,
						sensorAcelerometro,                                 
						SensorManager.SENSOR_DELAY_NORMAL);
			else  
				return false; 
		    
			    
		    // Sensor de campo magntico 
		    Sensor sensorMagnetico = sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);
		    if (sensorMagnetico != null)	
		      sensorManager.registerListener(escuchadorMagnetico,
		    		  sensorMagnetico,                                 
		    		  SensorManager.SENSOR_DELAY_NORMAL);
		    else 
		      return false;

		    // Estn presentes los dos sensores.
		    // Cabe destacar que el sensor de campo magntico
		    // en general no es muy exacto y puede haber grandes
		    // variaciones entre los distintos dispositivos
		    return true;
		  
	}

	// Implementacin de los escuchadores
	// de los sensores
	private final SensorEventListener escuchadorAcelerometro= new SensorEventListener() {
		public void onAccuracyChanged(Sensor sensor, int accuracy) { }
  
		public void onSensorChanged(SensorEvent event) {
			if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
				valoresAcelerometro = event.values;
				// Ver si ya estn los valores de 
				// los dos sensores
				sensor1 = true; 
				calcularOrientacion(); 

			};
		}
	};
  
 
  
	private final SensorEventListener escuchadorMagnetico = new SensorEventListener() {
  
		public void onAccuracyChanged(Sensor sensor, int accuracy) { }
  
		public void onSensorChanged(SensorEvent event) {
			if (event.sensor.getType() == Sensor.TYPE_MAGNETIC_FIELD) {
      	  		valoresCampoMagnetico = event.values;
      	  		// Ver si ya estn los valores de 
      	  		// los dos sensores
      	  		sensor2 = true; 
      	  		calcularOrientacion();    	
			};
		}
	};
  
	private static void calcularOrientacion() {
	  float[] R = new float[9];  
	  // Si ya hay valores de los dos sensores
	  // calcular la orientacin
	  if (sensor1 && sensor2) {
		  // Calcula la matriz en base a los valores de
		  // los dos sensores
		  SensorManager.getRotationMatrix(R, null, valoresAcelerometro, valoresCampoMagnetico);	  
	
		  // Obtiene la orientacin
		  SensorManager.getOrientation(R, values); 
	  }
	  
	}
  
 
  // Al reiniciarse la actividad
  // se vuelven a registrar los
  // escuchadores de los sensores
  @Override
  protected void onResume() {
    super.onResume();
    
    if (!comprobarSensores())
    	observaciones.setText("No se dispone de los sensores necesarios.");   
   
  }

  // Este cdigo se ejecuta dentro del mismo subproceso de
  // la interfaz de usuario
  private void actualizarPantalla() {
    runOnUiThread(new Runnable() {
      public void run() {
    	  // Comprueba que ya haya valores  
    	  // calculados
    	  if (Float.isNaN(values[0])) {
    		  azimuth.setText("");
        	  pitch.setText("");
        	  roll.setText("");
              observaciones.setText("No hay lectura correcta");
          }
    	  else {   	  
    		  // Actualiza la interfaz
    		  azimuth.setText(convGrados(values[0]) + " grados");
    		  pitch.setText(convGrados(values[1]) + " grados");
    		  roll.setText(convGrados(values[2]) + " grados");
    		  Time time = new Time();
    		  time.setToNow();    		  
    		  observaciones.setText("Hora: " + time.hour+":"+time.minute+":"+time.second);  
    	  }
    	  azimuth.invalidate();
    	  pitch.invalidate();
    	  roll.invalidate();
    	  observaciones.invalidate();
       
      }
    });
  };
  
  
  private String convGrados(float valor) {
	  // Convierte radianes a grados
	  float grados = (float) Math.toDegrees(valor);
	  // Muestra slo 2 decimales
      return String.format("%.2f", grados);
  }
  
  @Override
  protected void onPause() {
	// En cada pausa se anula el registro de los 
	// escuchadores
    sensorManager.unregisterListener(escuchadorAcelerometro);
    sensorManager.unregisterListener(escuchadorMagnetico);

    super.onPause();
  }
}