package com.inforbooks.smsmonitor;

import java.util.ArrayList;

import android.os.Bundle;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.telephony.SmsManager;
import android.view.Menu;
import android.widget.Toast;

public class MainActivity extends Activity {

	// Define los identificadores
	// para los objetos Intent
	String ENVIADO = "SMS_ENVIADO";
	String ENTREGADO = "SMS_ENTREGADO";

	// Objetos PendingIntent
	// para realizar los seguimientos
	// de envos y de entregas
	PendingIntent oIntentEnviado;
	PendingIntent oIntentEntregado;

	// BroadcastReceiver para atender
	// las notificaciones
	BroadcastReceiver oBroadcastEnviados;
	BroadcastReceiver oBroadcastEntregados;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		// Crea los 2 objetos PendingIntent
		oIntentEnviado = PendingIntent.getBroadcast(this, 0,
				new Intent(ENVIADO),
				PendingIntent.FLAG_UPDATE_CURRENT);

		oIntentEntregado = PendingIntent.getBroadcast(this, 0,
				new Intent(ENTREGADO),
				PendingIntent.FLAG_UPDATE_CURRENT);
	}

	@Override
	public void onResume() {
		super.onResume();

		// Se crean los objetos BroadcastReceiver
		// y se implementan los mtodos onReceive

		// BroadcastReceiver para controlar
		// y hacer el seguimiento de los
		// mensajes SMS enviados
		oBroadcastEnviados = new BroadcastReceiver(){
			@Override
			public void onReceive(Context arg0, Intent arg1) {

				String resultado = "?";

				switch (getResultCode())
				{
				case Activity.RESULT_OK:
					resultado =  "Envo correcto!";
					break;
				case SmsManager.RESULT_ERROR_GENERIC_FAILURE:
					resultado =  "Error no especfico. Mensaje no enviado.";
					break;
				case SmsManager.RESULT_ERROR_NO_SERVICE:
					resultado =  "No hay servicio. Mensaje no enviado.";
					break;
				case SmsManager.RESULT_ERROR_NULL_PDU:
					resultado =  "Error en PDU. Mensaje no enviado.";
					break;
				case SmsManager.RESULT_ERROR_RADIO_OFF:
					resultado =  "Error, radio no activa. Mensaje no enviado.";
					break;
				default:
					resultado =  "Error; cdigo no esperado.";
					break;

				}
				// Emite el mensaje con el resultado
				Toast.makeText(getBaseContext(), resultado, Toast.LENGTH_LONG).show();
			}
		};

		// BroadcastReceiver para controlar
		// la entrega de los mansajes SMS 
		// (en el emulador este Intent no se activa
		// nunca: probarlo en un dispositivo real)
		oBroadcastEntregados = new BroadcastReceiver(){
			@Override
			public void onReceive(Context arg0, Intent arg1) {

				Toast.makeText(getBaseContext(), "SMS entregado al receptor.",
						Toast.LENGTH_SHORT).show();

			}
		}; 

		// Se registran los objetos BroadcastReceiver 
		registerReceiver(oBroadcastEnviados, new IntentFilter(ENVIADO));
		registerReceiver(oBroadcastEntregados, new IntentFilter(ENTREGADO));      

		// Envo del mensaje 
		SmsManager oSM = SmsManager.getDefault();
		
		// Si el mensaje es largo se puede
		// fraccionar
		// simulamos un mensaje largo
		String mensajeLargo = "";
		for (int i=0; i < 12; i++ )
			mensajeLargo += "Mensaje largo ...";
		
		// Se crea un array para dividir el mensaje
		ArrayList<String> mensajePartido = oSM.divideMessage(mensajeLargo);
		
		// Se crean dos array que
		// despus tendrn la misma
		// cantidad de elementos
		// que la divisin del mensaje
		ArrayList<PendingIntent> intentsEnvios =new ArrayList<PendingIntent>();
		ArrayList<PendingIntent> intentsEntregas =new ArrayList<PendingIntent>();
		
		for (int i = 0; i < mensajePartido.size(); i++) {
			intentsEnvios.add(oIntentEnviado);
			intentsEntregas.add(oIntentEntregado);
		}
		
		// para usar el emulador utilizamos un AVD
		oSM.sendMultipartTextMessage("5554", 
				null, 
				mensajePartido, 
				intentsEnvios, 
				intentsEntregas);        
	}

	@Override
	public void onPause() {
		super.onPause();
		// Se anula el registro de los objetos BroadcastReceiver 
		// porque la actividad pasa a segundo plano
		unregisterReceiver(oBroadcastEnviados);
		unregisterReceiver(oBroadcastEntregados);        
	}



	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

}
