package com.inforbooks.videorecorder;

import java.io.IOException;

import android.app.Activity;
import android.hardware.Camera;
import android.media.CamcorderProfile;
import android.media.MediaRecorder;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.Menu;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;

public class MainActivity extends Activity implements SurfaceHolder.Callback {
  
	private static final String TAG = "VideoRecorder";
 
	private Camera oCamara;
	private SurfaceHolder oSurfaceHolder;
	private MediaRecorder oMR;
	private Boolean estaGrabando = false; 
  
  	@Override
  	public void onCreate(Bundle savedInstanceState) {
  		super.onCreate(savedInstanceState);
  		setContentView(R.layout.activity_main);  
  		
  		
  		// Se obtienen los objetos de la interfaz
  		Button grabar = (Button)findViewById(R.id.btnGrabar);
  		SurfaceView superficie = (SurfaceView)findViewById(R.id.surfaceView);
  		
  		
  		SurfaceHolder holder = superficie.getHolder();
  		// Asigna el mtodo de callback
  		// para saber cuando la superficie ya
  		// se haya creado
  		holder.addCallback(this);  		 
  		holder.setFixedSize(400, 300);  
  		
 
  		// Se define un listener para el botn de la interfaz
  		grabar.setOnClickListener(new OnClickListener() {
  			public void onClick(View v) {
  				if (estaGrabando) 
  					detenerGrabacion(); 					
  				else 
  					iniciarGrabacion(); 
  				
  			}    
  		});  

  	}
  
  
  	private void iniciarGrabacion() {
  		// Se encierra en un bloque try
  		// porque es probable que al
  		// iniciar la grabacin 
  		// haya una excepcin.
  		try {
	      oMR.start();
	      estaGrabando = true;
	      setText();
	    } catch (IllegalStateException e) {
	    	// Si no se puede iniciar
	    	// la grabacin, se libera
	    	// el objeto MediaRecorder
	    	// y se bloquea la cmara
	    	oMR.release();
	    	oCamara.lock();
	    	Log.d(TAG, "Illegal State Exception", e);
	    }
	}
  
  	private void detenerGrabacion() {
	    // Al finalizar la grabacin
	  	// se deben liberar los recursos
	    oMR.stop();
	    estaGrabando = false;
	    setText();	    
	        
	    // Reset y release para liberar
	    // los recursos de MediaRecorder.
	    oMR.reset();
	    oMR.release();
	    
	    // Se bloquea la cmara 
	    oCamara.lock();
	    
	    try {
	    	// se deja preparada la grabacin
	    	prepararVideoRecorder();
	    } catch (IllegalStateException e) {
	    	Log.e(TAG, "Illegal State Exception:", e);
	    } catch (IOException e) {
	    	Log.e(TAG, "Excepcin E/S:", e);
	    }
	}
  
  	private void setText(){
  		// Se obtienen los objetos de la interfaz
  		Button grabar = (Button)findViewById(R.id.btnGrabar); 
  		// Se coloca el texto vigente
  		if (estaGrabando)
  			grabar.setText(R.string.texto_detener);
  		else
  			grabar.setText(R.string.texto_grabar);
  		
  	}
	  
	@Override
	protected void onResume() {
		super.onResume();
    	oCamara = Camera.open();        
    
    	// Obtiene los parmetros de la cmara
    	// para indicar que slo se grabar
    	// audio/vdeo. Es decir, no se capturan
    	// imgenes fijas. Esto hace que la 
    	// inicializacin sea ms rpida.
    	Camera.Parameters parametros = oCamara.getParameters();
    	//parametros.setRecordingHint(true);
    	oCamara.setParameters(parametros);
	}
  
	@Override
	protected void onPause() {
		super.onPause();
		// Libera los recursos de MediaRecorder.
		oMR.reset();
		oMR.release();
		oCamara.lock();
    
		// Libera la cmara.
		oCamara.release();
	}
  
  
	private void prepararVideoRecorder() throws IllegalStateException, IOException {
		// Preparacin del objeto VideoRecorder  
	 
		// Crea una instancia de MediaRecorder
		oMR = new MediaRecorder();
    
  
		// Desbloquea la cmara para poder asignarla a MediaRecorder
		oCamara.unlock();   
		oMR.setCamera(oCamara);

		// Configura las fuentes de entrada 
		oMR.setAudioSource(MediaRecorder.AudioSource.CAMCORDER);
		oMR.setVideoSource(MediaRecorder.VideoSource.CAMERA);
		
		// Configuracin depende del nivel de la API
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO ){
			// El uso de perfiles requiere API 8 o superior
			
			// Intenta asignar la mejor calidad de resolucin vertical
			oMR.setProfile(CamcorderProfile.get(CamcorderProfile.QUALITY_HIGH));  
	 
		}
		else {
			// En versiones anteriores a la API 8 se establece
			// el formato y la codificacin sin utilizar
			// perfiles 
			oMR.setAudioEncoder(MediaRecorder.AudioEncoder.DEFAULT);
			oMR.setVideoEncoder(MediaRecorder.VideoEncoder.DEFAULT);
			oMR.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4);
			
		}
		 
		// Especifica el archivo de salida
		//oMR.setOutputFile(Environment.getExternalStorageDirectory().getPath() + "/testvideorecorder.mp4");
		String path = Environment.getExternalStorageDirectory().getAbsolutePath();
	    String    mArchivo = path + "/test.mp4";
		oMR.setOutputFile(mArchivo);

		// Muestra la vista previa
		oMR.setPreviewDisplay(oSurfaceHolder.getSurface());
		
		// Preparar para realizar la grabacin
		oMR.prepare();
	}  
  
	// Mtodos de implementacin para la gestin de SurfaceHolder  
	public void surfaceCreated(SurfaceHolder holder) { 
		oSurfaceHolder = holder;
		// La superficie se ha creado
		// ya se puede preparar 
		// el video grabador
		try {
			prepararVideoRecorder();
		} catch (IllegalStateException e) {
			Log.e(TAG, "Excepcin en surfaceCreated : Estado ilegal", e);
		} catch (IOException e) {
			Log.e(TAG, "Excepcin en surfaceCreated : E/S", e);
		}
	}
  
	public void surfaceDestroyed(SurfaceHolder holder) {
		oSurfaceHolder = null;
	}
  
	public void surfaceChanged(SurfaceHolder holder, int format, 
                             int width, int height) {
	}

 
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

}
