package com.inforbooks.notesderivada;


import android.content.Context;
import android.content.res.Resources;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.widget.TextView;

// Una vista creada por derivacin de la clase TextView
public class TextViewList extends TextView {

	// Objetos necesarios para dibujar en el canvas
	private int 	colorFondo; // color de fondo de la lista
	private float 	dimenSangria; // sangra del margen 
	private Paint 	paintMargen; // lnea del margen
	private Paint 	paintSeparador; // separador de elementos 
	private Paint 	paintCuadricula; // cuadriculado del fondo
	
	
	// mtodos constructores
	public TextViewList (Context context, AttributeSet ats, int ds) {
	    super(context, ats, ds);
	    // mtodo de inicializacin comn
	    inicializar();
	  }

	  public TextViewList (Context context) {
	    super(context);
	    inicializar();
	  }

	  public TextViewList (Context context, AttributeSet attrs) {
	    super(context, attrs);
	    inicializar();
	  }
	  
	  private void inicializar() {
		    // Objetos recursos 
		    Resources recursos = getResources();
		    
		    // Crea un pincel para la lnea separadora
		    // de elementos de la lista
		    paintSeparador = new Paint(Paint.ANTI_ALIAS_FLAG);
		    paintSeparador.setColor(recursos.getColor(R.color.notes_separador));
		    paintSeparador.setStrokeWidth(10);

		    // Crea un pincel para la lnea del margen
		    paintMargen = new Paint(Paint.ANTI_ALIAS_FLAG);
		    // Asigna el color del pincel
		    paintMargen.setColor(recursos.getColor(R.color.notes_margen));	   
		   
		    
		    // Crea un pincel para el cuadriculado interior de
		    // los elementos de la lista
		    paintCuadricula = new Paint(Paint.ANTI_ALIAS_FLAG);
		    paintCuadricula.setColor(recursos.getColor(R.color.notes_rejilla));

		    // Obtiene el color para el fondo de los
		    // elementos de la lista
		    colorFondo = recursos.getColor(R.color.notes_background);
		    
		    // Define la sangra para el texto
		    dimenSangria = recursos.getDimension(R.dimen.notes_sangria);
		  }
	  
	  @Override
	  public void onDraw(Canvas canvas) {
	    // Define el color de fondo de 
		// cada elemento de la lista
	    canvas.drawColor(colorFondo);

	    // Lneas separadoras de elementos
	    // que ocupa todo el ancho de la lista
	    canvas.drawLine(0, getMeasuredHeight(),
	                       getMeasuredWidth(), getMeasuredHeight(),
	                       paintSeparador);
	    
	    // Dibuja la cuadrcula del fondo
	    
	    // calcula cada cuntos pxeles traza las lneas
	    // Trazar las lneas horizontales
	    int incremento = getMeasuredHeight() / 6;
	    for (int pos=1; pos <=6; pos++){
	    	 canvas.drawLine(0, incremento * pos,
	    	                       getMeasuredWidth(), incremento * pos,
	    	                       paintCuadricula);
	    }
	    // Trazar las lneas verticales
	    for (int pos=incremento; pos <=getMeasuredWidth(); pos=pos+incremento){
	    	    	 canvas.drawLine(pos, 0,
	    	    	                       pos, getMeasuredHeight(),
	    	    	                       paintCuadricula);
	    	
	    }
	    
	    // Dibujar la lnea del margen
	    canvas.drawLine(dimenSangria, 0, dimenSangria, getMeasuredHeight(), paintMargen);

	    // Usa la sangra para colocar el texto
	    canvas.save();
	    canvas.translate(dimenSangria, 0);

	    // Usa la clase base para producir el texto
	    super.onDraw(canvas);
	    canvas.restore();
	  }

}
